;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Yoshi Love Minigame - Kamek, by Anonymous
;;
;; Description: This is a sprite of Kamek that comes out of a pipe, yells at
;; Mario, ducks back in, and begins tossing Yoshi Eggs out of a pipe.
;;
;; Uses first extra bit: YES
;; If the first extra bit is set, Kamek will set the first extra bit of the
;; spawned Baby Yoshis.  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!RAM_CURRENT_FRAME = $0DA1
!RAM_CURRENT_TILE = $5C
!RAM_CURRENT_ANIMATION = $79
!RAM_ANIMATION_FRAME = $7C
!RAM_ANIMATION_FRAME_COUNTER = $87
!RAM_ITEMS_COLLECTED = $60
!RAM_ITEMS_SPAWNED = $61
!RAM_ITEM_SPAWN_COUNTER = $62
!RAM_KAMEK_FLAGS = $0D9C; In order from left to right, has initially taunted, is taunting, is ducking into pipe, is tossing Yoshis out of pipe, is waiting for hat to fall back down, is coming out of pipe burnt, is shaking off soot, is coming out of pipe normal
!RAM_KAMEK_ACTION_COUNTER = $58
!RAM_HAS_YOSHI = $1864
;!RAM_NUMBER_OF_YOSHIS = $0DD4

!RAM_SPRITE_DIRECTION = $0DD4

!KAMEK_SPRITE_TO_THROW = #$01 ;The sprite that Kamek will throw (set this to the sprite ID of babyyoshi_yl.asm)

!KAMEK_SPRITE_PRIZE = #$02 ;The sprite that Kamek will spawn when the game is
;over. This sprite needs to make Mario leave the stage, or he'll be stuck in 
;the bonus room forever.

!RAM_SPAWNED_PRIZE_ID_TABLE = $1528;

!KAMEK_SPRITE_PALETTE = #$00 ; The palette to spawn the Baby Yoshis in when not using the extra bit. Useful if you're using custom palettes or don't wish to have the Baby Yoshis spawn in four random colors.

!KAMEK_USE_PIPE_SFX = #$04
!KAMEK_USE_PIPE_SFX_BANK = $1DF9

!KAMEK_TAUNT_SFX = #$22
!KAMEK_TAUNT_SFX_BANK = $1DFC

!KAMEK_PRIZE_SFX = #$29
!KAMEK_PRIZE_SFX_BANK = $1DFC

!RAM_SPAWNED_SPRITE_DIRECTION_TABLE = $C2;

print "INIT ",pc
LDA $187A
STA !RAM_HAS_YOSHI
STZ $187A
STZ $76 ;Set the player to looking left
STZ !RAM_CURRENT_FRAME
STZ !RAM_CURRENT_TILE
STZ !RAM_CURRENT_ANIMATION
STZ !RAM_ANIMATION_FRAME
STZ !RAM_KAMEK_ACTION_COUNTER
;STZ !RAM_NUMBER_OF_YOSHIS
LDA $14
STA !RAM_ANIMATION_FRAME_COUNTER
LDA #%00000001
STA !RAM_KAMEK_FLAGS

RTL

print "MAIN ",pc

LDA #$12
STA $7E0EFF
LDA #$1D
STA $7E0F00
LDA #$0E
STA $7E0F01
LDA #$16
STA $7E0F02

LDA !RAM_ITEMS_COLLECTED
AND #%00001111
STA $7E0F1D
LDA !RAM_ITEMS_COLLECTED
LSR A
LSR A
LSR A
LSR A
STA $7E0F1C
LDA #$FC
STA $7E0F03
STA $7E0F04
STA $7E0F1E
STA $7E0F1F
LDA #$FC
STA $7E0F1B

MainRoutine:
LDA !RAM_KAMEK_FLAGS
CMP #%00000001
BEQ KamekLeavingPipeNormal
CMP #%01000000
BEQ KamekTaunting
CMP #%00100000
BEQ KamekEnteringPipe
CMP #%00010000
BNE CheckIfKamek
JMP KamekTossingYoshis
CheckIfKamek:
CMP #%00010000
BNE KamekGraphics
JMP KamekIdle

KamekLeavingPipeNormal:
LDA #$0B
STA $71
LDA #$F0
STA $AA,x
JSL $01802A
LDA !RAM_KAMEK_ACTION_COUNTER
CMP #$00
BNE ContinueLeavingPipeNormal
LDA !KAMEK_USE_PIPE_SFX
STA !KAMEK_USE_PIPE_SFX_BANK
LDA !RAM_KAMEK_ACTION_COUNTER
ContinueLeavingPipeNormal:
INC A
STA !RAM_KAMEK_ACTION_COUNTER
CMP #$20
BCC KamekGraphics
STZ !RAM_KAMEK_ACTION_COUNTER
LDA #%01000000
STA !RAM_KAMEK_FLAGS
BRA KamekGraphics

KamekTaunting:
LDA !RAM_KAMEK_ACTION_COUNTER
CMP #$00
BNE ContinueTaunting
LDA !KAMEK_TAUNT_SFX
STA !KAMEK_TAUNT_SFX_BANK
LDA #$01
;STA !RAM_CURRENT_ANIMATION
JSR ChangeAnimation
LDA !RAM_KAMEK_ACTION_COUNTER
ContinueTaunting:
INC A
STA !RAM_KAMEK_ACTION_COUNTER
CMP #$60
BCC KamekGraphics
STZ !RAM_KAMEK_ACTION_COUNTER
LDA #%00100000
STA !RAM_KAMEK_FLAGS
LDA #$00
JSR ChangeAnimation
BRA KamekGraphics

KamekEnteringPipe:
LDA #$0F
STA $AA,x
JSL $01802A
LDA !RAM_KAMEK_ACTION_COUNTER
CMP #$00
BNE ContinueEnteringPipe
LDA !KAMEK_USE_PIPE_SFX
STA !KAMEK_USE_PIPE_SFX_BANK
LDA !RAM_KAMEK_ACTION_COUNTER
ContinueEnteringPipe:
INC A
STA !RAM_KAMEK_ACTION_COUNTER
CMP #$20
BCC KamekGraphics
;STZ !RAM_KAMEK_ACTION_COUNTER
LDA #%00010000
STA !RAM_KAMEK_FLAGS
LDA #$00
STA $71
LDA #$20
STA !RAM_KAMEK_ACTION_COUNTER
BRA KamekGraphics

KamekGraphics:

JSR Animate

JSR Graphics
RTL

KamekTossingYoshis:
LDA #$00
STA $71

LDA !RAM_KAMEK_ACTION_COUNTER
CMP #$00
BNE DecrementYoshiCounter

JSL $02A9DE
BMI SlotsFull ; If sprite slots are full, don't generate.

LDA !RAM_ITEMS_SPAWNED
CMP #$50
BCC DontDespawn

CMP #$51
BEQ DespawnKamek

INC !RAM_ITEMS_SPAWNED
LDA #$80
STA !RAM_KAMEK_ACTION_COUNTER
RTL

DespawnKamek:

JSR SpawnPrize

STZ $14C8,x

;LDA #$44
;STA $14AB
RTL

DontDespawn:

;PHX ; Push current sprite.
;TYX ; Get sprite to generate into X.
;JSL $07F7D2 ; Reset sprite tables, to generate this one.
;JSL $0187A7 ; Reset custom sprite tables, too.
;PLX ; Restore previous sprite.

;After this, the sprite to generate is in the Y register.

PHX ; NOTE: We need to use the X register for the generated sprite.
TYX ; So we preserve the current sprites data.

JSL $07F7D2 ; Reset sprite tables, to generate this one.
JSL $0187A7 ; Reset custom sprite tables, too.

;LDA #$01
LDA !KAMEK_SPRITE_TO_THROW
STA $7FAB9E,x ; The sprite number is stored to the custom sprite RAM Address.

;LDA #$08
;STA $7FAB10,x ; Setting this to 08 means we are generating a custom sprite.

TXY ; Transfer sprite to spawn back to Y.
PLX ; Restore X.

;LDA #$08
;STA $7FAB10,x ; Setting this to 08 means we are generating a custom sprite.

LDA #$01
STA $14C8,y ; Run sprite's INIT code first.

;LDA $E4,x
;SEC
;SBC #$08
JSL $01ACF9
LDA $148E
AND #%00000011
CMP #$03
BEQ SpawnPos3
CMP #$02
BEQ SpawnPos2

BRA SpawnPos1

ContinueSpawningSprite:

PHX ; Push current sprite.
TYX ; Get sprite to generate into X.
JSL $07F7D2 ; Reset sprite tables, to generate this one.
JSL $0187A7 ; Reset custom sprite tables, too.
LDA #$08
STA $7FAB10,x ; Setting this to 08 means we are generating a custom sprite.
LDA !RAM_SPRITE_DIRECTION
STA !RAM_SPAWNED_SPRITE_DIRECTION_TABLE,x
PLX ; Restore previous sprite.

LDA #$20
;LDA #$FF
STA !RAM_KAMEK_ACTION_COUNTER

LDA !RAM_ITEMS_SPAWNED
AND #%00001111
CMP #$09
BCC IncreaseOne

LDA !RAM_ITEMS_SPAWNED
CLC
ADC #$07
STA !RAM_ITEMS_SPAWNED
BRA SlotsFull

IncreaseOne:
INC !RAM_ITEMS_SPAWNED

SlotsFull:
RTL

DecrementYoshiCounter:
LDA !RAM_KAMEK_ACTION_COUNTER
DEC
STA !RAM_KAMEK_ACTION_COUNTER
RTL

SpawnPos1:
LDA #$17
STA $00E4,y ; X position.
LDA $14E0,x
STA $14E0,y ; X position high.
;LDA $D8,x
;SEC
;SBC #$10
;SBC #$30
LDA #$86
STA $00D8,y ; Y position.
LDA $14D4,x
STA $14D4,y ; Y position high.
LDA #$00
STA !RAM_SPRITE_DIRECTION
JMP ContinueSpawningSprite

SpawnPos2:
LDA #$78
STA $00E4,y ; X position.
LDA $14E0,x
STA $14E0,y ; X position high.
;LDA $D8,x
;SEC
;SBC #$10
;SBC #$30
LDA #$E0
STA $00D8,y ; Y position.
;LDA $14D4,x
LDA #$00
STA $14D4,y ; Y position high.
LDA #$01
STA !RAM_SPRITE_DIRECTION
BRA ContinueSpawningSprite

SpawnPos3:
LDA #$D8
STA $00E4,y ; X position.
LDA $14E0,x
STA $14E0,y ; X position high.
;LDA $D8,x
;SEC
;SBC #$10
;SBC #$30
LDA #$86
STA $00D8,y ; Y position.
LDA $14D4,x
STA $14D4,y ; Y position high.
LDA #$00
STA !RAM_SPRITE_DIRECTION
JMP ContinueSpawningSprite

KamekIdle:

RTL

SpawnPrize:

JSL $02A9DE
BMI SlotsFull ; If sprite slots are full, don't generate.

PHX ; NOTE: We need to use the X register for the generated sprite.
TYX ; So we preserve the current sprites data.

JSL $07F7D2 ; Reset sprite tables, to generate this one.
JSL $0187A7 ; Reset custom sprite tables, too.

LDA !KAMEK_SPRITE_PRIZE
STA $7FAB9E,x ; The sprite number is stored to the custom sprite RAM Address.

TXY ; Transfer sprite to spawn back to Y.
PLX ; Restore X.



LDA #$01
STA $14C8,y ; Run sprite's INIT code first.

LDA #$78
STA $00E4,y ; X position.
LDA $14E0,x
STA $14E0,y ; X position high.
LDA #$18
STA $00D8,y ; Y position.
LDA $14D4,x
STA $14D4,y ; Y position high.

LDA !RAM_ITEMS_COLLECTED
STA $58

LDA !KAMEK_PRIZE_SFX
STA !KAMEK_PRIZE_SFX_BANK

PHX ; Push current sprite.
TYX ; Get sprite to generate into X.
JSL $07F7D2 ; Reset sprite tables, to generate this one.
JSL $0187A7 ; Reset custom sprite tables, too.
LDA #$08
STA $7FAB10,x ; Setting this to 08 means we are generating a custom sprite.
PLX ; Restore previous sprite.

LDA #$26
STA $7E0F1C
LDA #$FC
STA $7E0F1D
LDA #$64
STA $7E0F1B

RTS

;===================================
; ANIMATION TABLES HERE
;===================================

NUMFRAMES: 
db %00010001,%00110011
STARTFRAME:
db $00,$01,$00,$00
ANIMATION:
db $00,$01,$02
FRAMEDURATION:
db $FF,$05,$05

;===================================
; ANIMATION ROUTINE HERE
;===================================
Animate:

PHX

LDX !RAM_CURRENT_ANIMATION
LDA STARTFRAME,x
CLC
ADC !RAM_ANIMATION_FRAME
TAX

LDA $14
SBC !RAM_ANIMATION_FRAME_COUNTER
CMP FRAMEDURATION,x
BCS ChangeFrame

PLX
RTS

ChangeFrame:
LDA $14
STA !RAM_ANIMATION_FRAME_COUNTER
PHX
LDA !RAM_CURRENT_ANIMATION
LSR A
TAX
LDA !RAM_CURRENT_ANIMATION
AND #%00000001
CMP #$00
BEQ BitShiftAnimationIDRight
LDA NUMFRAMES,x
AND #%00001111
BRA CheckCurrentFrame

BitShiftAnimationIDRight:
LDA NUMFRAMES,x
LSR A
LSR A
LSR A
LSR A

CheckCurrentFrame:
PLX
CMP !RAM_ANIMATION_FRAME
BCC ResetAnimation
CMP !RAM_ANIMATION_FRAME
BEQ ResetAnimation

LDA !RAM_ANIMATION_FRAME
INC A
STA !RAM_ANIMATION_FRAME

TXA
INC A
TAX

LDA ANIMATION,x
STA !RAM_CURRENT_FRAME

PLX

RTS


;PLX

ResetAnimation:
LDX !RAM_CURRENT_ANIMATION
LDA STARTFRAME,x
TAX
LDA ANIMATION,x
STA !RAM_CURRENT_FRAME
STZ !RAM_ANIMATION_FRAME

PLX

RTS

ChangeAnimation:
STA !RAM_CURRENT_ANIMATION
;TAX
;PHA
;LDA STARTFRAME,x
;TAX
;LDA ANIMATION,x
;STA !RAM_CURRENT_FRAME
;STZ !RAM_ANIMATION_FRAME
;LDA $14
;STA !RAM_ANIMATION_FRAME_COUNTER
;PLA
RTS


;===================================
; GRAPHICS MAPS HERE
;===================================

NUMTILES: 
db %00110011,%00110011
STARTTILE: 
db $00,$04,$08,$00
TILE: 
db $27,$01,$20,$30,$21,$01,$20,$30,$23,$01,$20,$30
TILEX: 
db $FC,$FC,$F4,$F4,$FC,$FC,$F4,$F4,$FC,$FC,$F4,$F4
TILEY: 
db $00,$F0,$00,$08,$00,$F0,$00,$08,$00,$F0,$00,$08
TILESIZE: 
db %00110011,%00110011,%00110011,%00110011
TILEPROP: 
db %00000111,%00000111,%00000111,%00000111,%00000111,%00000111,%00000111,%00000111,%00000111,%00000111,%00000111,%00000111

;===================================
; GRAPHICS ROUTINE HERE
;===================================
Graphics:
;STZ !RAM_CURRENT_TILE
JSR GET_DRAW_INFO

;Store the X Register in the stack and load it with the number of tiles to draw
PHX
LDA !RAM_CURRENT_FRAME
LSR A
TAX
LDA !RAM_CURRENT_FRAME
AND #%00000001
CMP #$00
BEQ BitShiftFrameIDRight
LDA NUMTILES,x
AND #%00001111
TAX
BRA TileDrawingLoop

BitShiftFrameIDRight:
LDA NUMTILES,x
LSR A
LSR A
LSR A
LSR A
TAX

;Now that the X register has the number of tiles to draw, start drawing them!
TileDrawingLoop:
PHX
LDX !RAM_CURRENT_FRAME
LDA !RAM_CURRENT_TILE
CLC
ADC STARTTILE,x
;LDA STARTTILE,x
TAX

LDA $00
CLC
ADC TILEX,x
;ADC !RAM_CURRENT_TILE
STA $0300,y

LDA $01
CLC
ADC TILEY,x
STA $0301,y

LDA TILE,x
STA $0302,y

LDA TILEPROP,x
;ORA $64
STA $0303,y

TYA
LSR A
LSR A
PHY
TAY

PHX
TXA
LSR A
LSR A
LSR A
TAX
LDA TILESIZE,x
TAX

PLA
AND #%00000111
TileCountLoop:
CMP #$00
BEQ GetTileSizeBit
DEC A
PHX
TAX
PLA
LSR A
PHX
TAX
PLA
BRA TileCountLoop
GetTileSizeBit:
TXA
AND #%00000001
CMP #$00
BEQ StoreTileSize
ASL A
StoreTileSize:

;LDA #$02       ;02 = 16x16, 00 = 8x8
STA $0460,y    ;store to size table
PLY

INY
INY
INY
INY

PLA

CMP !RAM_CURRENT_TILE
BEQ EndTileDrawingLoop

TAX

;LDA !RAM_CURRENT_TILE
INC !RAM_CURRENT_TILE
;STA !RAM_CURRENT_TILE

BRA TileDrawingLoop

EndTileDrawingLoop:

LDY #$FF ; This means the tile drawn is 16x16.
;LDA #$00 ; The number of tiles I wrote - 1.

PLX

JSL $01B7B3

STZ !RAM_CURRENT_TILE

RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;       Y = index to sprite OAM ($300)
;       $00 = sprite x position relative to screen boarder
;       $01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1:              db $0C,$1C
SPR_T2:              db $01,$02

GET_DRAW_INFO:       STZ $186C,x             ; reset sprite offscreen flag, vertical
	STZ $15A0,x             ; reset sprite offscreen flag, horizontal
	LDA $E4,x               ; \
	CMP $1A                 ;  | set horizontal offscreen if necessary
	LDA $14E0,x             ;  |
	SBC $1B                 ;  |
	BEQ ON_SCREEN_X         ;  |
	INC $15A0,x             ; /

ON_SCREEN_X:         LDA $14E0,x             ; \
	XBA	 ;  |
	LDA $E4,x               ;  |
	REP #$20                ;  |
	SEC	 ;  |
	SBC $1A                 ;  | mark sprite invalid if far enough off screen
	CLC	 ;  |
	ADC.W #$0040            ;  |
	CMP.W #$0180            ;  |
	SEP #$20                ;  |
	ROL A                   ;  |
	AND #$01                ;  |
	STA $15C4,x             ; / 
	BNE INVALID             ; 
	
	LDY #$00                ; \ set up loop:
	LDA $1662,x             ;  | 
	AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
	BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
	INY	 ; / 
ON_SCREEN_LOOP:      LDA $D8,x               ; \ 
	CLC	 ;  | set vertical offscreen if necessary
	ADC SPR_T1,y            ;  |
	PHP	 ;  |
	CMP $1C                 ;  | (vert screen boundry)
	ROL $00                 ;  |
	PLP	 ;  |
	LDA $14D4,x             ;  | 
	ADC #$00                ;  |
	LSR $00                 ;  |
	SBC $1D                 ;  |
	BEQ ON_SCREEN_Y         ;  |
	LDA $186C,x             ;  | (vert offscreen)
	ORA SPR_T2,y            ;  |
	STA $186C,x             ;  |
ON_SCREEN_Y:         DEY	 ;  |
	BPL ON_SCREEN_LOOP      ; /

	LDY $15EA,x             ; get offset to sprite OAM
	LDA $E4,x               ; \ 
	SEC	 ;  | 
	SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
	STA $00                 ; / 
	LDA $D8,x               ; \ 
	SEC	 ;  | 
	SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
	STA $01                 ; / 
	RTS	 ; return

INVALID:             PLA	 ; \ return from *main gfx routine* subroutine...
	PLA	 ;  |    ...(not just this subroutine)
	RTS	 ; /
