;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Yoshi Love Minigame - Baby Yoshi, by Anonymous
;;
;; Description: This is a sprite of Baby Yoshi, that is designed to be spawned 
;; out of a pipe by Kamek. It will bounce off a set sprite (assumed to be a spring), and Mario needs to keep it from hitting the ground.
;;
;; Uses first extra bit: YES
;; If the first extra bit is set, the Baby Yoshi's color will be randomized
;; between the default palette's green, red, yellow, and blue palettes.  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!RAM_FLOAT_TIMER_TABLE = $163E
!RAM_SMOKE_TIMER_TABLE = $1540
!RAM_SPAWNED_SPRITE_DIRECTION_TABLE = $C2
!RAM_SPAWNED_SPRITE_ID = $1528
!RAM_ALREADY_CHANGED_ID = $58
!RAM_HAS_YOSHI = $1864; Set this to the same RAM value as in Kamek's sprite

!FLOAT_TIME = #$40

!ONEUP_VALUE = #$20
!TWOUP_VALUE = #$30
!THREEUP_VALUE = #$40

!LIVES_SOUND = #$29
!NOTHING_SOUND = #$2A
!LIVES_SOUND_ADDRESS = $1DFC
!NOTHING_SOUND_ADDRESS = $1DFC
!END_MUSIC = #$09

;===================================
; GRAPHIC TABLES HERE
;===================================

PRIZEATTRIBS: 
db %00001011,%00001010,%00000100,%00000101
PRIZETILE:
db $44,$24,$24,$48
ONEUPATTRIBS: 
db %00001011,%00001010,%00000100,%00000100
ONEUPTILE:
db $56,$56,$29,$39
SMOKETILE:
db $00,$66,$66,$64,$64,$62,$62,$60,$60

print "INIT ",pc
LDA #$00
STA !RAM_SPAWNED_SPRITE_DIRECTION_TABLE,x
LDA !FLOAT_TIME
STA !RAM_FLOAT_TIMER_TABLE,x
LDA #$08
STA !RAM_SMOKE_TIMER_TABLE,x
LDA !RAM_ALREADY_CHANGED_ID
CMP !THREEUP_VALUE
BCS THREE
CMP !TWOUP_VALUE
BCS TWO
CMP !ONEUP_VALUE
BCS ONE
LDA #$00
BRA SETID
ONE:
LDA #$01
BRA SETID
TWO:
LDA #$02
BRA SETID
THREE:
LDA #$03
BRA SETID
SETID:
STA !RAM_ALREADY_CHANGED_ID
RTL

print "MAIN ",pc

LDA !RAM_FLOAT_TIMER_TABLE,x
CMP #$00
BNE BusinessAsUsual
LDA !RAM_SPAWNED_SPRITE_DIRECTION_TABLE,x
CMP #$00
BEQ DirToOne
LDA #$00
STA !RAM_SPAWNED_SPRITE_DIRECTION_TABLE,x
LDA !FLOAT_TIME
STA !RAM_FLOAT_TIMER_TABLE,x
BRA BusinessAsUsual
DirToOne:
LDA #$01
STA !RAM_SPAWNED_SPRITE_DIRECTION_TABLE,x
LDA !FLOAT_TIME
STA !RAM_FLOAT_TIMER_TABLE,x
BusinessAsUsual:
LDA !RAM_SPAWNED_SPRITE_DIRECTION_TABLE,x
CMP #$00
BNE FloatDown
LDA #$FE
BRA MoveAndCheckCollisions
FloatDown:
LDA #$02

MoveAndCheckCollisions:
STA $AA,x
JSL $01802A

JSL $01A7DC
BCC NoContact

;LDA $0DBE
;CLC
;ADC !RAM_SPAWNED_SPRITE_ID,x
;STA $0DBE

LDA !RAM_ALREADY_CHANGED_ID
STA $18E4

LDA !RAM_ALREADY_CHANGED_ID
CMP #$01
BCS PlayLivesSound

LDA !NOTHING_SOUND
STA !NOTHING_SOUND_ADDRESS

EndBonus:
LDA !END_MUSIC
STA $1DFB

LDA #$0B
STA $71

LDA !RAM_HAS_YOSHI
STA $187A

LDA #$FF
STA $1493

STZ $14C8,x

RTL

PlayLivesSound:
LDA !LIVES_SOUND
STA !LIVES_SOUND_ADDRESS

BRA EndBonus

NoContact:

JSR GET_DRAW_INFO

JMP GraphicRoutine

GraphicRoutine:

;===================================
; GRAPHICS ROUTINE HERE
;===================================

PHX

LDX !RAM_ALREADY_CHANGED_ID

LDA $00
STA $0300,y

LDA $01
STA $0301,y

LDA PRIZETILE,x
STA $0302,y

LDA PRIZEATTRIBS,x
STA $0303,y

TYA
LSR A
LSR A
PHY
TAY
LDA #$02       ;02 = 16x16, 00 = 8x8
STA $0460,y    ;store to size table
PLY

LDA !RAM_ALREADY_CHANGED_ID
CMP #$01
BCC CheckForSmoke

INY
INY
INY
INY

LDA $00
STA $0300,y

LDA $01
CLC
ADC #$10
STA $0301,y

LDA ONEUPTILE,x
STA $0302,y

LDA ONEUPATTRIBS,x
;ORA $64
STA $0303,y

TYA
LSR A
LSR A
PHY
TAY
LDA #$00      ;02 = 16x16, 00 = 8x8
STA $0460,y    ;store to size table
PLY

INY
INY
INY
INY

LDA $00
CLC
ADC #$08
STA $0300,y

LDA $01
CLC
ADC #$10
STA $0301,y

LDA #$57
STA $0302,y

LDA ONEUPATTRIBS,x
;ORA $64
STA $0303,y

TYA
LSR A
LSR A
PHY
TAY
LDA #$00       ;02 = 16x16, 00 = 8x8
STA $0460,y    ;store to size table
PLY

INY
INY
INY
INY

CheckForSmoke:

PLX

LDA !RAM_SMOKE_TIMER_TABLE,x
CMP #$00
BEQ EndGraphics

PHX

TAX

LDA $00
STA $0300,y

LDA $01
STA $0301,y

LDA SMOKETILE,x
STA $0302,y

LDA 00111010
STA $0303,y

PLX

TYA
LSR A
LSR A
PHY
TAY
LDA #$02       ;02 = 16x16, 00 = 8x8
STA $0460,y    ;store to size table
PLY

INY
INY
INY
INY

EndGraphics:

LDY #$FF ; This means the tile drawn is 16x16.
LDA !RAM_SMOKE_TIMER_TABLE,x
CMP #$00
BNE CheckSmoke
LDA !RAM_ALREADY_CHANGED_ID
CMP #$01
BCS DrawAllTiles
LDA #$00
BRA ContinueT
DrawAllTiles:
LDA #$02 ; The number of tiles I wrote - 1.

ContinueT:
JSL $01B7B3

RTL

CheckSmoke:
LDA !RAM_ALREADY_CHANGED_ID
CMP #$01
BCS DrawAllTilesSmoke
LDA #$01
BRA ContinueT
DrawAllTilesSmoke:
LDA #$03 ; The number of tiles I wrote - 1.
BRA ContinueT

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;       Y = index to sprite OAM ($300)
;       $00 = sprite x position relative to screen boarder
;       $01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1:              db $0C,$1C
SPR_T2:              db $01,$02

GET_DRAW_INFO:       STZ $186C,x             ; reset sprite offscreen flag, vertical
	STZ $15A0,x             ; reset sprite offscreen flag, horizontal
	LDA $E4,x               ; \
	CMP $1A                 ;  | set horizontal offscreen if necessary
	LDA $14E0,x             ;  |
	SBC $1B                 ;  |
	BEQ ON_SCREEN_X         ;  |
	INC $15A0,x             ; /

ON_SCREEN_X:         LDA $14E0,x             ; \
	XBA	 ;  |
	LDA $E4,x               ;  |
	REP #$20                ;  |
	SEC	 ;  |
	SBC $1A                 ;  | mark sprite invalid if far enough off screen
	CLC	 ;  |
	ADC.W #$0040            ;  |
	CMP.W #$0180            ;  |
	SEP #$20                ;  |
	ROL A                   ;  |
	AND #$01                ;  |
	STA $15C4,x             ; / 
	BNE INVALID             ; 
	
	LDY #$00                ; \ set up loop:
	LDA $1662,x             ;  | 
	AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
	BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
	INY	 ; / 
ON_SCREEN_LOOP:      LDA $D8,x               ; \ 
	CLC	 ;  | set vertical offscreen if necessary
	ADC SPR_T1,y            ;  |
	PHP	 ;  |
	CMP $1C                 ;  | (vert screen boundry)
	ROL $00                 ;  |
	PLP	 ;  |
	LDA $14D4,x             ;  | 
	ADC #$00                ;  |
	LSR $00                 ;  |
	SBC $1D                 ;  |
	BEQ ON_SCREEN_Y         ;  |
	LDA $186C,x             ;  | (vert offscreen)
	ORA SPR_T2,y            ;  |
	STA $186C,x             ;  |
ON_SCREEN_Y:         DEY	 ;  |
	BPL ON_SCREEN_LOOP      ; /

	LDY $15EA,x             ; get offset to sprite OAM
	LDA $E4,x               ; \ 
	SEC	 ;  | 
	SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
	STA $00                 ; / 
	LDA $D8,x               ; \ 
	SEC	 ;  | 
	SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
	STA $01                 ; / 
	RTS	 ; return

INVALID:             PLA	 ; \ return from *main gfx routine* subroutine...
	PLA	 ;  |    ...(not just this subroutine)
	RTS	 ; /
