;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Yoshi Love Minigame - Baby Yoshi, by Anonymous
;;
;; Description: This is a sprite of Baby Yoshi, that is designed to be spawned 
;; out of a pipe by Kamek. It will bounce off a set sprite (assumed to be a spring), and Mario needs to keep it from hitting the ground.
;;
;; Uses first extra bit: YES
;; If the first extra bit is set, the Baby Yoshi's color will be randomized
;; between the default palette's green, red, yellow, and blue palettes.  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; MAKE SURE THAT FOR EACH OF THESE VALUES THERE IS AT LEAST 3 BYTES OF FREE 
;; RAM AFTER THE STARTING BYTE. THERE WILL BE UP TO FOUR BABY YOSHIS ACTIVE AT 
;; ANY GIVEN TIME.

!RAM_TO_INCREASE = $60

!RAM_BABY_YOSHI_ID = $0F62

!RAM_DIRECTION_TABLE = $C2; This is the sprite table to set the item's
; direction. Set this to one of SMW's sprite tables that doesn't automatically
; decrement or to any free RAM with at least 12 bytes available.

!SPRITE_SPRING = #$2F; This is the sprite the Baby Yoshi's will bounce off of. Assumed to be a spring.

print "INIT ",pc
RTL

print "MAIN ",pc

LDA !RAM_DIRECTION_TABLE,x
CMP #$01
BEQ MoveUp2
LDA #$F0
BRA ContinueMoving
MoveUp2:
LDA #$10
ContinueMoving:
STA $AA,x
JSL $01802A

JSL $01A7DC
BCC NoContact

LDA !RAM_TO_INCREASE
AND #%00001111
CMP #$09
BCC IncreaseOne

LDA !RAM_TO_INCREASE
CLC
ADC #$07
STA !RAM_TO_INCREASE
BRA SoundEffect

IncreaseOne:
INC !RAM_TO_INCREASE

SoundEffect:
LDA #$01 ; Some sound effect
STA $1DFC

IsAboveScreen:
STZ $14C8,x

RTL

NoContact:

JSR GET_DRAW_INFO

LDA !RAM_DIRECTION_TABLE,x
CMP #$00
BEQ MoveUp

LDA $01
CLC
ADC #$10
CMP #$00
BEQ IsAboveScreen
BCC IsAboveScreen
BRA GraphicRoutine


MoveUp:
LDA $01
SEC
SBC #$10
CMP #$F0
BCS IsAboveScreen

GraphicRoutine:

;===================================
; GRAPHICS ROUTINE HERE
;===================================

LDA $00
STA $0300,y

LDA $01
STA $0301,y

LDA #$0E
STA $0302,y

LDA #%00000101
;ORA $64
STA $0303,y

TYA
LSR A
LSR A
PHY
TAY
LDA #$02       ;02 = 16x16, 00 = 8x8
STA $0460,y    ;store to size table
PLY

LDA $13
LSR
LSR
LSR
AND #$01
CMP #$00
BEQ ContinueGFX

JMP NoTarget

ContinueGFX:

INY
INY
INY
INY

LDA $00
SEC
SBC #$06
STA $0300,y

LDA $01
CLC
ADC #$10
STA $0301,y

LDA #$40
STA $0302,y

LDA #%00000101
;ORA $64
STA $0303,y

TYA
LSR A
LSR A
PHY
TAY
LDA #$00      ;02 = 16x16, 00 = 8x8
STA $0460,y    ;store to size table
PLY

INY
INY
INY
INY

LDA $00
CLC
ADC #$02
STA $0300,y

LDA $01
CLC
ADC #$10
STA $0301,y

LDA #$41
STA $0302,y

LDA #%00000101
;ORA $64
STA $0303,y

TYA
LSR A
LSR A
PHY
TAY
LDA #$00       ;02 = 16x16, 00 = 8x8
STA $0460,y    ;store to size table
PLY

INY
INY
INY
INY

LDA $00
CLC
ADC #$0A
STA $0300,y

LDA $01
CLC
ADC #$10
STA $0301,y

LDA #$42
STA $0302,y

LDA #%00000101
;ORA $64
STA $0303,y

TYA
LSR A
LSR A
PHY
TAY
LDA #$00       ;02 = 16x16, 00 = 8x8
STA $0460,y    ;store to size table
PLY

INY
INY
INY
INY

LDA $00
CLC
ADC #$12
STA $0300,y

LDA $01
CLC
ADC #$10
STA $0301,y

LDA #$43
STA $0302,y

LDA #%00000101
;ORA $64
STA $0303,y

TYA
LSR A
LSR A
PHY
TAY
LDA #$00       ;02 = 16x16, 00 = 8x8
STA $0460,y    ;store to size table
PLY

INY
INY
INY
INY

LDY #$FF ; This means the tile drawn is 16x16.
LDA #$04 ; The number of tiles I wrote - 1.

JSL $01B7B3

RTL

NoTarget:
LDY #$FF ; This means the tile drawn is 16x16.
LDA #$00 ; The number of tiles I wrote - 1.

JSL $01B7B3

RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;       Y = index to sprite OAM ($300)
;       $00 = sprite x position relative to screen boarder
;       $01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1:              db $0C,$1C
SPR_T2:              db $01,$02

GET_DRAW_INFO:       STZ $186C,x             ; reset sprite offscreen flag, vertical
	STZ $15A0,x             ; reset sprite offscreen flag, horizontal
	LDA $E4,x               ; \
	CMP $1A                 ;  | set horizontal offscreen if necessary
	LDA $14E0,x             ;  |
	SBC $1B                 ;  |
	BEQ ON_SCREEN_X         ;  |
	INC $15A0,x             ; /

ON_SCREEN_X:         LDA $14E0,x             ; \
	XBA	 ;  |
	LDA $E4,x               ;  |
	REP #$20                ;  |
	SEC	 ;  |
	SBC $1A                 ;  | mark sprite invalid if far enough off screen
	CLC	 ;  |
	ADC.W #$0040            ;  |
	CMP.W #$0180            ;  |
	SEP #$20                ;  |
	ROL A                   ;  |
	AND #$01                ;  |
	STA $15C4,x             ; / 
	BNE INVALID             ; 
	
	LDY #$00                ; \ set up loop:
	LDA $1662,x             ;  | 
	AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
	BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
	INY	 ; / 
ON_SCREEN_LOOP:      LDA $D8,x               ; \ 
	CLC	 ;  | set vertical offscreen if necessary
	ADC SPR_T1,y            ;  |
	PHP	 ;  |
	CMP $1C                 ;  | (vert screen boundry)
	ROL $00                 ;  |
	PLP	 ;  |
	LDA $14D4,x             ;  | 
	ADC #$00                ;  |
	LSR $00                 ;  |
	SBC $1D                 ;  |
	BEQ ON_SCREEN_Y         ;  |
	LDA $186C,x             ;  | (vert offscreen)
	ORA SPR_T2,y            ;  |
	STA $186C,x             ;  |
ON_SCREEN_Y:         DEY	 ;  |
	BPL ON_SCREEN_LOOP      ; /

	LDY $15EA,x             ; get offset to sprite OAM
	LDA $E4,x               ; \ 
	SEC	 ;  | 
	SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
	STA $00                 ; / 
	LDA $D8,x               ; \ 
	SEC	 ;  | 
	SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
	STA $01                 ; / 
	RTS	 ; return

INVALID:             PLA	 ; \ return from *main gfx routine* subroutine...
	PLA	 ;  |    ...(not just this subroutine)
	RTS	 ; /
