;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Press Your Luck styled bonus game, by Dotsarecool
;;
;; Thanks imamelia for the disassembly of the invisible solid block, which was used as a base.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; defines and tables
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!block_offsets = $30 ; distance between the green and red block

XOffsetLo:
		db $0E,$F1,$10,$E0,$1F,$F1
XOffsetHi:
		db $00,$FF,$00,$FF,$00,$FF

		
; these are the 4 boards that are cycled on screen. 0-9 = 0-90 coins, A = 1up, B = 3up, C = mushroom, D = fire, E = cape, F = poison mushroom
; padded with 2 bytes of FF at the end of each line
boards:
		db $03,$0C,$0A,$0F,$04,$06,$0F,$0D,$00,$03,$04,$0F,$0E,$02,$FF,$FF
		db $05,$0F,$02,$00,$03,$0E,$0B,$0F,$01,$0D,$02,$03,$0F,$04,$FF,$FF
		db $0F,$04,$07,$0E,$0A,$03,$0F,$07,$03,$0F,$04,$0C,$00,$0D,$FF,$FF
		db $0E,$09,$0F,$05,$0F,$01,$00,$03,$0D,$05,$0F,$0A,$06,$03,$FF,$FF
		
; the x,y position of each of the tiles that make up the board
x_positions:
		db $48,$60,$78,$90,$A8,$A8,$A8,$A8,$90,$78,$60,$48,$48,$48
y_positions:
		db $28,$28,$28,$28,$28,$40,$58,$70,$70,$70,$70,$70,$58,$40
		
; the oam tile and property bytes for the 16 different prizes
tiles:
		db $62,$A0,$A2,$A4,$A6,$A8,$AA,$AC,$AE,$C0,$24,$80,$24,$26,$0E,$82
properties:
		db $30,$35,$35,$35,$35,$35,$35,$35,$35,$35,$3A,$35,$38,$3A,$34,$37
		
; the oam tile and property bytes for the different powerups
item_tiles:
		db $FF,$24,$26,$FF,$0E
item_properties:
		db $FF,$38,$3A,$FF,$34


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
		STZ $1504,X ; which blocks are hit this frame
		STZ $1510,X ; frame counter for board/cursor switching
		STZ $151C,X ; number of coins / 10 to give to player
		STZ $1528,X ; number of lives to give player
		STZ $1534,X ; powerup to give player
		STZ $1558,X ; timer for how long to wait until done pausing
		STZ $160E,X ; current position of the cursor
		STZ $1626,X ; current submode of the sprite
		LDA $13 ; seed the RNG with the frame counter
		STA $148B
		LDA $14
		STA $148C
		RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine wrapper
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "MAIN ",pc
main:
		PHB
		PHK
		PLB
		LDA #$00
		PHA
		LDA $94
		PHA
		SEC
		SBC #!block_offsets
		STA $94
		JSR InvisibleSolidBlockMain ; carry set = right block hit
		PLA
		STA $94
		BCC .second_block
		; first block hit
		PLA
		ORA #$01
		PHA
	.second_block:
		JSR InvisibleSolidBlockMain ; carry set = left block hit
		PLA
		BCC .process
		; second block hit
		ORA #$02
	.process:
		STA $1504,X ; $1504,X = ------lr
		JSR PressYourLuckMain
		JSR DrawEverything
		PLB
		RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

InvisibleSolidBlockMain:

		PHK		;
		PER $0006	;
		PEA $8020	;
		JML $01A7F7	; process interaction between the player and the sprite
		BCC NoContact	; return if there was no contact

		LDA $D8,x	; sprite Y position low byte
		SEC		;
		SBC $1C		; minus screen boundary Y position low byte
		STA $00		; save the result

		LDA $80		; player Y position relative to the screen
		CLC		;
		ADC #$18	; + 18
		CMP $00		; if the player is within a certain range of the sprite...
		BPL SecondCheck	; skip to the second part of the code

		LDA $7D		; if the player is moving upward...
		BMI NoContact	;
		LDA $77		; or if the player is touching the ceiling...
		AND #$08	;
		BNE NoContact	; there was no contact

		LDA #$10		;
		STA $7D		; set the player Y speed to 10
		LDA #$01		;
		STA $1471	; and set the "on a sprite" flag

		LDA #$1F		; base Y offset: 1F
		LDY $187A	; if the player is on Yoshi...
		BEQ StoreYOffset	;
		LDA #$2F		; then the Y offset is 2F
StoreYOffset:	;
		STA $01		;
		LDA $D8,x	; sprite Y position low byte
		SEC		;
		SBC $01		; minus the offset
		STA $96		; into the player Y position low byte
		LDA $14D4,x	; sprite Y position high byte
		SBC #$00		; handle overflow
		STA $97		; into the player Y position high byte

		LDA $77		;
		AND #$03	; if the player is touching a wall...
		BNE IsContact	; there is still contact, but don't move the player with the sprite
		LDY #$00		; start Y at 00
		LDA $1528,x	; $1528,x holds the number of pixels to move the player when riding a sprite
		BPL ShiftRight	; if it is positive, the sprite and player are moving to the right
		DEY		;
ShiftRight:	;
		CLC		;
		ADC $94		; add this value to the player's X position
		STA $94		;
		TYA		; and handle the high byte
		ADC $95		;
		STA $95		;

IsContact:

		;SEC		; set the carry flag, indicating that there was contact between the player and the sprite
		CLC
		RTS		;

NoContact:
		CLC		; clear the carry flag, indicating that there was no contact
		RTS		;

SecondCheck:

		LDA #$00		; A = 00
		LDY $73		; if the player is ducking...
		BNE Offset08	;
		LDY $19		; or small...
		BNE SetOffset	;
Offset08:		; then the offset initial value is 08
		LDA #$08		;
SetOffset:		;
		LDY $187A	; if the player is on Yoshi...
		BEQ NoAdd8	;
		ADC #$08	; then add 8 more to the offset value
NoAdd8:		;
		CLC		;
		ADC $80		; add the player's Y position relative to the screen
		CMP $00		; if the result is less than the value from the sprite Y position/screen boundary...
		BCC Offset3	; jump to the third part of code

		LDA $7D		; if the player is moving down...
		BPL NoContact2	; there will be no contact
		LDA #$10		;
		STA $7D		; if not, set the player's Y speed to 10

NotQBlock:	;

		LDA #$01		;
		STA $1DF9	; play the "hit head on solid surface" sound effect
		SEC
		RTS

NoContact2:

		CLC		;
		RTS		;

Offset3:

		JSR SubHorzPos

Index0:

		LDA XOffsetLo,y	;
		CLC		;
		ADC $E4,x	; set up the low byte of the X offset
		STA $94		;
		LDA XOffsetHi,y	; set up the high byte
		ADC $14E0,x	;
		STA $95		; shift the player horizontally
		STZ $7B		; and set his/her X speed to zero
		CLC		;
		RTS		;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; subroutines
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SubHorzPos:

		LDY #$00
		LDA $94
		SEC
		SBC $E4,x
		STA $0F
		LDA $95
		SBC $14E0,x
		BPL $01
		INY
		RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PressYourLuckMain:
		PHX
		LDA $1626,X
		ASL A
		TAX
		JMP (.sprite_submodes,X)
	
	.sprite_submodes:
		dw .cycling
		dw .pausing
		dw .finished
		
	.cycling:
		PLX
		LDA $1504,X
		BIT #$01
		BEQ .dont_quit
		LDA #$02
		STA $1626,X
		LDA #$44
		STA $14AB
		LDA #$80
		STA $1558,X
		RTS
	.dont_quit:
		BIT #$02
		BEQ .dont_trigger
		INC $1626,X
		LDA #$80
		STA $1558,X
		RTS
	.dont_trigger:
		INC $1510,X
		LDA $1510,X
		AND #$0F
		BNE .skip
		LDA #$23 ; play beep sound
		STA $1DFC
	.try_again:
		JSL $01ACF9 ; RNG
		AND #$0F
		CMP #$0E
		BCC .no_wrap
		SEC
		SBC #$0E
	.no_wrap:
		CMP $160E,X
		BEQ .try_again
		STA $160E,X
	.skip:
		RTS
		
		
	.pausing:
		PLX
		LDA $1558,X
		BEQ .restart
		CMP #$78
		BEQ .play_sound
		CMP #$30
		BEQ .add_total
		RTS
	.restart:
		STZ $1626,X
		LDA $1510,X
		AND #$C0
		CLC
		ADC #$3F
		STA $1510,X
		RTS
		
	.play_sound:
		LDA $1510,X
		AND #$C0
		LSR #2
		STA $00
		LDA $160E,X
		ORA $00
		TAY
		LDA boards,Y
		BEQ .done_sound
		CMP #$0F
		BNE .good_prize
		LDA #$2A ; buzz
		STA $1DFC
		BRA .done_sound
	.good_prize:
		LDA #$29 ; ding
		STA $1DFC
	.done_sound:
		RTS
		
	.add_total:
		LDA $1510,X
		AND #$C0
		LSR #2
		STA $00
		LDA $160E,X
		ORA $00
		TAY
		LDA boards,Y
		CMP #$0F
		BEQ .you_lose
		PHA
		LDA $1558,X
		CMP #$30
		BNE .done
		PLA
		CMP #$0A
		BCC .add_coins
		CMP #$0C
		BCC .add_lives
	.add_item:
		CMP #$0C
		BEQ .give_mushroom
		CMP #$0D
		BEQ .give_flower
	.give_feather:
		LDA #$04
		BRA .done_item
	.give_flower:
		LDA #$02
		BRA .done_item
	.give_mushroom:
		LDA #$01
		BRA .done_item
	.done_item:
		STA $1534,X
		RTS		
	.add_coins:
		CLC
		ADC $151C,X
		CMP #$0A
		BCC .no_coin_overflow
		INC $1528,X
		PHA
		JSR .check_max_lives ; (puke)
		PLA
		SEC
		SBC #$0A
	.no_coin_overflow:
		STA $151C,X
		RTS
	.add_lives:
		INC $1528,X
		CMP #$0B
		BNE .check_max_lives
		INC $1528,X
		INC $1528,X
	.check_max_lives:
		LDA $1528,X
		CMP #$64
		BCC .done_lives
		LDA #$63
		STA $1528,X
	.done_lives:
		RTS	
	.done:
		PLA
		RTS	
	.you_lose:
		STZ $151C,X
		STZ $1528,X
		STZ $1534,X
		LDA #$02
		STA $1626,X
		LDA #$44
		STA $14AB
		RTS
		
		
	.finished:
		PLX
		LDA $1558,X
		CMP #$60
		BEQ .cash_out_lives
		CMP #$40
		BEQ .cash_out_coins
		CMP #$20
		BEQ .cash_out_item
		RTS
	.cash_out_lives:
		LDA $151C,X
		PHX
		JSL $00974C ; hex2dec
		STX $00
		PLX
		STA $151C,X
		LDA $1528,X
		CLC
		ADC $00
		STA $18E4
		RTS
	.cash_out_coins:
		LDA $151C,X
		ASL #2
		CLC
		ADC $151C,X
		ASL A
		STA $13CC
		RTS
	.cash_out_item:
		LDA $1534,X
		BEQ .no_cash_out_item
		STA $0DC2
		LDA #$0B ; get item sound
		STA $1DFC
	.no_cash_out_item:
		RTS

DrawEverything:
		JSR draw_left_block
		JSR draw_right_block
		JSR draw_panel
		JSR draw_cursor
		JSR draw_counters
		RTS

draw_left_block:
		PHP
		REP #$20
		LDA $E4,X ; sprite x pos
		AND #$00FF
		SEC
		SBC $1A ; layer 1 x pos
		CMP #$FFF0
		BCS .continue
		CMP #$0100
		BCS .done
	.continue:
		STA $0324 ; oam x pos
		SEP #$20
		XBA
		AND #$01
		ORA #$02 ; large tile
		STA $0469 ; oam tile size
		REP #$20
		LDA $D8,X ; sprite y pos
		AND #$00FF
		SEC
		SBC $1C ; layer 1 y pos
		SEP #$20
		STA $0325 ; oam y pos
		LDA #$40
		STA $0326 ; oam tile
		LDA #$3A
		STA $0327 ; oam properties
	.done:
		PLP
		RTS

draw_right_block:
		PHP
		REP #$20
		LDA $E4,X ; sprite x pos
		CLC
		ADC.W #!block_offsets
		AND #$00FF
		SEC
		SBC $1A ; layer 1 x pos
		CMP #$FFF0
		BCS .continue
		CMP #$0100
		BCS .done
	.continue:
		STA $0328 ; oam x pos
		SEP #$20
		XBA
		AND #$01
		ORA #$02 ; large tile
		STA $046A ; oam tile size
		REP #$20
		LDA $D8,X ; sprite y pos
		AND #$00FF
		SEC
		SBC $1C ; layer 1 y pos
		SEP #$20
		STA $0329 ; oam y pos
		LDA #$40
		STA $032A ; oam tile
		LDA #$38
		STA $032B ; oam properties
	.done:
		PLP
		RTS

draw_panel:
		LDY #$0D
	.loop:
		PHX
		LDA $1510,X
		AND #$C0
		LSR #2
		STA $00
		TYA
		ORA $00
		TAX
		LDA boards,X
		STA $00
		
		PHY
		TYA
		ASL #2
		TAX
		LDA $00
		TAY
		LDA tiles,Y ; tile
		STA $0342,X
		LDA properties,Y ; properties
		STA $0343,X
		PLY
		LDA #$02 ; size
		STA $0470,Y
		LDA x_positions,Y
		STA $0340,X ; x pos
		LDA y_positions,Y
		STA $0341,X ; y pos
		
		PLX
		DEY
		BPL .loop
		RTS

draw_cursor:
		LDA $1626,X
		BEQ .draw
		CMP #$02
		BEQ .done
		LDA $14
		AND #$04
		BEQ .draw
	.done:
		RTS
	.draw:
		LDA #$84 ; tile
		STA $0382
		STA $0386
		STA $038A
		STA $038E
		LDA #$33 ; properties
		STA $0383
		LDA #$73
		STA $0387
		LDA #$B3
		STA $038B
		LDA #$F3
		STA $038F
		LDA #$02 ; size
		STA $0480
		STA $0481
		STA $0482
		STA $0483
		LDA $160E,X ; x pos
		TAY
		LDA x_positions,Y
		PHA
		SEC
		SBC #$08
		STA $0380
		STA $0388
		PLA
		CLC
		ADC #$08
		STA $0384
		STA $038C
		LDA $160E,X ; y pos
		TAY
		LDA y_positions,Y
		PHA
		SEC
		SBC #$08
		STA $0381
		STA $0385
		PLA
		CLC
		ADC #$08
		STA $0389
		STA $038D		
		RTS

draw_counters:
		JSR draw_coins
		JSR draw_lives
		JSR draw_item
		RTS
		
draw_coins:
		LDA $151C,X
		BNE .draw
		RTS
	.draw:
		LDA #$E8
		STA $0336 ; tile
		LDA #$34
		STA $0337 ; properties
		LDA #$02
		STA $046D ; size
		LDA #$C0
		STA $0334 ; x pos
		LDA #$90
		STA $0335 ; y pos
		LDA $151C,X
		ORA #$F0
		STA $033A ; tile
		LDA #$F0
		STA $033E ; tile
		LDA #$31
		STA $033B ; properties
		STA $033F
		STZ $046E ; size
		STZ $046F
		LDA #$D8
		STA $0338 ; x pos
		LDA #$E0
		STA $033C
		LDA #$94
		STA $0339 ; y pos
		STA $033D
		
		RTS
		
draw_lives:
		LDA $1528,X
		BNE .draw
		RTS
	.draw:
		LDA #$24
		STA $0332 ; tile
		LDA #$3A
		STA $0333 ; properties
		LDA #$02
		STA $046C ; size
		LDA #$C0
		STA $0330 ; x pos
		LDA #$A0
		STA $0331 ; y pos
		PHX
		LDA $1528,X
		JSL $00974C ; hex2dec
		ORA #$F0
		STA $037E ; tile
		TXA
		ORA #$F0
		STA $037A ; tile
		LDA #$31
		STA $037B ; properties
		STA $037F
		STZ $047E ; size
		STZ $047F
		LDA #$D8
		STA $0378 ; x pos
		LDA #$E0
		STA $037C
		LDA #$A4
		STA $0379 ; y pos
		STA $037D
		
		PLX
		RTS
		
draw_item:
		LDA $1534,X
		BNE .draw
		RTS
	.draw:
		TAY
		LDA item_tiles,Y
		STA $032E ; tile
		LDA item_properties,Y
		STA $032F ; properties
		LDA #$02
		STA $046B ; size
		LDA #$C0
		STA $032C ; x pos
		LDA #$B0
		STA $032D ; y pos
		RTS