;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; INIT and MAIN JSL targets
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!existence_counter = $1540
!frame = $C2

!sprite_status = $14C8

!sprite_y_lo = $D8
!sprite_y_hi = $14D4

!sprite_x_lo = $E4
!sprite_x_hi = $14E0

!sprite_x_speed = $B6
!sprite_y_speed = $AA

!init_y_speed = #$C0

!existence_time = #$2C

!subsequent_hits_counter = $1FFF


                    print "INIT ",pc
                    PHB
                    PHK             
                    PLB
                    lda !init_y_speed
                    sta !sprite_y_speed,x
                    stz !sprite_x_speed,x
                    stz !frame
                    lda !existence_time
                    sta !existence_counter,x
                    plb
                    RTL
                    print "MAIN ",pc            
                    PHB
                    PHK             
                    PLB
                    JSR SPRITE_ROUTINE          ;Jump to the routine to keep organized
                    PLB
                    RTL     


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SPRITE_ROUTINE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


!frame_change_1 = #$24
!frame_change_2 = #$1C
!frame_change_3 = #$14
!frame_change_4 = #$0C

SPRITE_ROUTINE:     
                    lda $9D
                    bne return

                    lda !existence_counter,x
                    bne +
                        stz !sprite_status,x
                        stz !subsequent_hits_counter
                    +
                    cmp !frame_change_1
                    beq .do_frame_change
                    cmp !frame_change_2
                    beq .do_frame_change
                    cmp !frame_change_3
                    beq .do_frame_change
                    cmp !frame_change_4
                    beq .do_frame_change
                    bra .no_frame_change
                    .do_frame_change
                        inc !frame,x
                    .no_frame_change

                    JSL $01802A
                    JSL $01803A

                    lda !existence_counter,x
                    cmp #$05
                    bcs +
                        jsr interact_with_target
                    +

                    return:
                    JSR SUB_OFF_SCREEN_X0
                    JSR SUB_GFX                 ;Draw the graphics
                    RTS                         ;End the routine

                    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GRAPHICS ROUTINE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Tiles:
db $0B,$0D,$2B,$2D,$4D

SUB_GFX:            jsr GetDrawInfo         ;Get all the drawing info, duh
                    phx
                    lda !frame,x
                    tax

                    LDA $00
                    STA $0300,y
                    
                    LDA $01
                    STA $0301,y

                    LDA Tiles,x
                    STA $0302,y

                    PHX
                    LDX $15E9
                    LDA $15F6,x
                    PLX
                    ORA $64
                    ora #%00110000
                    STA $0303,y
                    plx

                    LDA #$00
                    LDY #$02
                    JSL $01B7B3                 ;/and then draw em
                    EndIt:
                    RTS

!target_sprite_num = #$00

interact_with_target:
     LDY #$0B
INTLOOP:  
     CPY #$00
     BNE SPR_CON
     JMP NO_SPR_CON
     SPR_CON:
     DEY
     STX $06
     CPY $06
     BEQ INTLOOP
     LDA $14C8,y    ; this just checks if
     CMP #$08  ; the sprite touching this
     BCC INTLOOP    ; is dead or not, if you want it to interact with dead sprites, remove 3 lines
     LDA $1686,y
     BIT #$08
     BNE INTLOOP
     JSL $03B69F
     PHX
     TYX
     JSL $03B6E5
     PLX
     JSL $03B72B
     BCC INTLOOP
     PHX
     TYX
     LDA $7FAB10,x
     BIT #$08
     BEQ Normal
     lda $7FAB9E,x
     cmp !target_sprite_num
     bne Normal

     lda #$0D
     sta !sprite_status,x

     PLX

     jsr DrawSmoke
     stz !sprite_status,x


     BRA NO_SPR_CON
     
     Normal:
     PLX

     JMP INTLOOP
NO_SPR_CON:
     RTS  


DrawSmoke:          LDY #$03                ; \ find a free slot to display effect
FINDFREE:           LDA $17C0,y             ;  |
                    BEQ FOUNDONE            ;  |
                    DEY                     ;  |
                    BPL FINDFREE            ;  |
                    RTS                     ; / return if no slots open

FOUNDONE:           LDA #$01                ; \ set effect graphic to smoke puff
                    STA $17C0,y             ; /
                    LDA #$18                ; \ set time to show smoke
                    STA $17CC,y             ; /
                    LDA $D8,x               ; \ smoke y position = generator y position
                    STA $17C4,y             ; /
                    LDA $E4,x               ; \ load generator x position and store it for later
                    STA $17C8,y             ; /
                    RTS  
                    
                
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;       Y = index to sprite OAM ($300)
;       $00 = sprite x position relative to screen boarder
;       $01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    
DATA_03B75C:        db $0C,$1C
DATA_03B75E:        db $01,$02

GetDrawInfo:        STZ $186C,X               ; Reset sprite offscreen flag, vertical 
                    STZ $15A0,X               ; Reset sprite offscreen flag, horizontal 
                    LDA $E4,X                 ; \ 
                    CMP $1A                   ;  | Set horizontal offscreen if necessary 
                    LDA $14E0,X               ;  | 
                    SBC $1B                   ;  | 
                    BEQ ADDR_03B774           ;  | 
                    INC $15A0,X               ; / 
ADDR_03B774:        LDA $14E0,X               ; \ 
                    XBA                       ;  | Mark sprite invalid if far enough off screen 
                    LDA $E4,X                 ;  | 
                    REP #$20                  ; Accum (16 bit) 
                    SEC                       ;  | 
                    SBC $1A                   ;  | 
                    CLC                       ;  | 
                    ADC.W #$0040              ;  | 
                    CMP.W #$0180              ;  | 
                    SEP #$20                  ; Accum (8 bit) 
                    ROL                       ;  | 
                    AND.B #$01                ;  | 
                    STA $15C4,X               ;  | 
                    BNE ADDR_03B7CF           ; /  
                    LDY.B #$00                ; \ set up loop: 
                    LDA $1662,X               ;  |  
                    AND.B #$20                ;  | if not smushed (1662 & 0x20), go through loop twice 
                    BEQ ADDR_03B79A           ;  | else, go through loop once 
                    INY                       ; /                        
ADDR_03B79A:        LDA $D8,X                 ; \                        
                    CLC                       ;  | set vertical offscree 
                    ADC DATA_03B75C,Y         ;  |                       
                    PHP                       ;  |                       
                    CMP $1C                   ;  | (vert screen boundry) 
                    ROL $00                   ;  |                       
                    PLP                       ;  |                       
                    LDA $14D4,X               ;  |                       
                    ADC.B #$00                ;  |                       
                    LSR $00                   ;  |                       
                    SBC $1D                   ;  |                       
                    BEQ ADDR_03B7BA           ;  |                       
                    LDA $186C,X               ;  | (vert offscreen)      
                    ORA DATA_03B75E,Y         ;  |                       
                    STA $186C,X               ;  |                       
ADDR_03B7BA:        DEY                       ;  |                       
                    BPL ADDR_03B79A           ; /                        
                    LDY $15EA,X               ; get offset to sprite OAM                           
                    LDA $E4,X                 ; \ 
                    SEC                       ;  |                                                     
                    SBC $1A                   ;  |                                                    
                    STA $00                   ; / $00 = sprite x position relative to screen boarder 
                    LDA $D8,X                 ; \                                                     
                    SEC                       ;  |                                                     
                    SBC $1C                   ;  |                                                    
                    STA $01                   ; / $01 = sprite y position relative to screen boarder 
                    RTS                       ; Return 

ADDR_03B7CF:        PLA                       ; \ Return from *main gfx routine* subroutine... 
                    PLA                       ;  |    ...(not just this subroutine) 
                    RTS                       ; / 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
SPR_T12:             db $40,$B0
SPR_T13:             db $01,$FF
SPR_T14:             db $30,$C0,$A0,$C0,$A0,$F0,$60,$90     ;bank 1 sizes
                    db $30,$C0,$A0,$80,$A0,$40,$60,$B0      ;bank 3 sizes
SPR_T15:             db $01,$FF,$01,$FF,$01,$FF,$01,$FF     ;bank 1 sizes
                    db $01,$FF,$01,$FF,$01,$00,$01,$FF      ;bank 3 sizes

SUB_OFF_SCREEN_X1:   LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SUB_OFF_SCREEN_X2:   LDA #$04                ;  | 
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X3:   LDA #$06                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X4:   LDA #$08                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X5:   LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X6:  LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X7:   LDA #$0E                ;  |
STORE_03:           STA $03                 ;  |            
                    BRA START_SUB           ;  |
SUB_OFF_SCREEN_X0:   STZ $03                    ; /

START_SUB:           JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ; | 
                    ADC #$50                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; | 
                    CMP #$02                ; | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13
                    AND #$01
                    ORA $03
                    STA $01
                    TAY
                    LDA $1A
                    CLC
                    ADC SPR_T14,y
                    ROL $00
                    CMP $E4,x
                    PHP
                    LDA $1B
                    LSR $00
                    ADC SPR_T15,y
                    PLP
                    SBC $14E0,x
                    STA $00
                    LSR $01
                    BCC SPR_L31
                    EOR #$80
                    STA $00
SPR_L31:             LDA $00
                    BPL RETURN_35
ERASE_SPRITE:        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /    
                    LDY $161A,x
                    CPY #$FF
                    BEQ KILL_SPRITE
                    LDA #$00
                    STA $1938,y
KILL_SPRITE:         STZ $14C8,x             ; erase sprite
RETURN_35:           RTS                     ; return

VERTICAL_LEVEL:      LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; | 
                    BCS RETURN_35           ; /
                    LDA $E4,x               ; \ 
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13
                    LSR A
                    AND #$01
                    STA $01
                    TAY
                    LDA $1C
                    CLC
                    ADC SPR_T12,y
                    ROL $00
                    CMP $D8,x
                    PHP
                    LDA.w $001D 
                    LSR $00
                    ADC SPR_T13,y
                    PLP
                    SBC $14D4,x
                    STA $00
                    LDY $01
                    BEQ SPR_L38
                    EOR #$80
                    STA $00
SPR_L38:             
                    LDA $00
                    BPL RETURN_35
                    BMI ERASE_SPRITE

SUB_IS_OFF_SCREEN:   LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ; |  
                    RTS                     ; / return

