;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; INIT and MAIN JSL targets
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!enemy_type = $C2
!pattern = $1510
!direction = $157C
!state = $1594
!countdown_timer = $1540

!sprite_status = $14C8

!sprite_y_lo = $D8
!sprite_y_hi = $14D4

!sprite_x_lo = $E4
!sprite_x_hi = $14E0

!x_speed = $B6
!y_speed = $AA

!cluster_sprite_x_pos_hi = $1E3E
!cluster_sprite_x_pos_lo = $1E16
!cluster_sprite_y_pos_hi = $1E2A
!cluster_sprite_y_pos_lo = $1E02

!cluster_sprite_y_speed = $1E52
!cluster_sprite_x_speed = $1E66
!cluster_sprite_x_speed_acc = $1E8E
!cluster_sprite_y_speed_acc = $0F72

!cluster_sprite_number = $1892
!coin_cluster_sprite_number = #$0B

!coin_time_to_live = #$40
!coin_alive_counter = $0F4A

!make_coin_sounds_count = $1DEF
!make_coin_sounds_counter = $1B97

!subsequent_hits_counter = $1FFF
!max_subsequent_hits = #$09

!get_random_number = $81ACF9

!sprite_spawn_x_lo = $0A
!sprite_spawn_x_hi = $0B
!sprite_spawn_y_lo = $0C
!sprite_spawn_y_hi = $0D
!sprite_spawn_x_speed = $0E
!sprite_spawn_y_speed = $0F
!sprite_spawn_return_slot = $09
!sprite_spawn_sprite_number = $7C


                    print "INIT ",pc
                    PHB
                    PHK             
                    PLB
                    phx
                    lda !pattern,x
                    asl
                    tax
                    rep #$20
                    lda init_routines,x
                    ldx #$00
                    sta $00
                    sep #$20
                    jsr ($0000,x)
                    plx
                    plb
                    RTL
                    print "MAIN ",pc            
                    PHB
                    PHK             
                    PLB
                    JSR SPRITE_ROUTINE          ;Jump to the routine to keep organized
                    PLB
                    RTL     


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SPRITE_ROUTINE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

XSpeeds:
db $10,$F0

SPRITE_ROUTINE:
                    lda $9D
                    bne return
                    lda !sprite_status,x
                    cmp.b #$0D
                    bne +
                        jmp kill_routine
                    +
                    phx
                    lda !pattern,x
                    asl
                    tax
                    rep #$20
                    lda movement_routines,x
                    ldx #$00
                    sta $00
                    sep #$20
                    jsr ($0000,x)
                    plx

                    JSL $01802A
                    JSL $01803A
                    return:
                    JSR SUB_OFF_SCREEN_X0
                    JSR SUB_GFX                 ;Draw the graphics
                    RTS                         ;End the routine


coin_inc_counts:
    db $01,$02,$03,$05,$08,$0A,$0F,$19,$32,$64
    
coin_spawn_counts:
    db $00,$01,$02,$04,$07,$09,$0C,$0C,$0C,$0C

coin_sound_count:
    db $01,$02,$03,$04,$04,$04,$04,$05,$06,$07

kill_routine:
    lda !subsequent_hits_counter
    phx
    tax
    lda coin_inc_counts,x
    clc
    adc $13CC
    sta $13CC

    lda coin_sound_count,x
    sta !make_coin_sounds_count

    ;Spawn score sprite or something

    lda coin_spawn_counts,x
    plx
    jsr spawn_coins

    stz !sprite_status,x
    lda !subsequent_hits_counter
    cmp.b !max_subsequent_hits
    bcs +
        inc !subsequent_hits_counter
    +
    rts
    

spawn_coins:
    pha
    lda !sprite_y_hi,x
    sta !sprite_spawn_y_hi
    lda !sprite_y_lo,x
    sta !sprite_spawn_y_lo
    lda !sprite_x_hi,x
    sta !sprite_spawn_x_hi
    lda !sprite_x_lo,x
    sta !sprite_spawn_x_lo
    lda #$C0
    sta !sprite_spawn_y_speed
    lda !coin_cluster_sprite_number
    sta !sprite_spawn_sprite_number
    pla
    tay
    .spawn_loop
        jsl !get_random_number
        lda #$00
        sta $4205
        lda $148D
        sta $4204
        lda.b #$20
        sta $4206
        nop #12
        lda $4216
        sec
        sbc.b #$10
        sta !sprite_spawn_x_speed
        jsr spawn_cluster_sprite
        phy
        ldy !sprite_spawn_return_slot
        lda !coin_time_to_live
        sta !coin_alive_counter,y
        lda #$00
        lda #$00
        sta !cluster_sprite_y_speed_acc,y
        sta !cluster_sprite_x_speed_acc,y
        ply
        dey
        bpl .spawn_loop
    rts


spawn_cluster_sprite:
    phy
    jsr get_cluster_sprite_slot
    bmi .dont_spawn
    lda !sprite_spawn_y_hi
    sta !cluster_sprite_y_pos_hi,y
    lda !sprite_spawn_y_lo
    sta !cluster_sprite_y_pos_lo,y
    lda !sprite_spawn_x_hi
    sta !cluster_sprite_x_pos_hi,y
    lda !sprite_spawn_x_lo
    sta !cluster_sprite_x_pos_lo,y

    lda !sprite_spawn_y_speed
    sta !cluster_sprite_y_speed,y
    lda !sprite_spawn_x_speed
    sta !cluster_sprite_x_speed,y

    lda !sprite_spawn_sprite_number
    sta !cluster_sprite_number,y

    sty !sprite_spawn_return_slot

    .dont_spawn
    ply
    rts

get_cluster_sprite_slot:
    ldy #19
    .get_slot_loop
        lda !cluster_sprite_number,y
        beq .got_slot
        dey
        bpl .get_slot_loop
    .got_slot
    rts

init_routines:
    dw init_bush_1_walk_routine
    dw init_bush_2_walk_routine
    dw init_platform_1_walk_routine
    dw init_platform_2_walk_routine
    dw init_platform_3_walk_routine



init_bush_1_walk_routine:
    ldx $15E9
    stz !state,x
    lda !direction,x
    asl
    phx
    tax
    rep #$20
    lda .start_x,x
    sta $00
    lda .start_y,x
    plx
    sep #$20
    sta !sprite_y_lo,x
    xba
    sta !sprite_y_hi,x
    lda $00
    sta !sprite_x_lo,x
    lda $01
    sta !sprite_x_hi,x
    rts

.start_x
    dw $0068,$0048
.start_y
    dw $0180,$0180

init_bush_2_walk_routine:
    ldx $15E9
    stz !state,x
    lda !direction,x
    asl
    phx
    tax
    rep #$20
    lda .start_x,x
    sta $00
    lda .start_y,x
    plx
    sep #$20
    sta !sprite_y_lo,x
    xba
    sta !sprite_y_hi,x
    lda $00
    sta !sprite_x_lo,x
    lda $01
    sta !sprite_x_hi,x
    rts

.start_x
    dw $00B8,$0098
.start_y
    dw $0180,$0180
init_platform_1_walk_routine:
    ldx $15E9
    stz !state,x
    lda !direction,x
    asl
    phx
    tax
    rep #$20
    lda .start_x,x
    sta $00
    lda .start_y,x
    plx
    sep #$20
    sta !sprite_y_lo,x
    xba
    sta !sprite_y_hi,x
    lda $00
    sta !sprite_x_lo,x
    lda $01
    sta !sprite_x_hi,x
    rts

.start_x
    dw $00E0,$00B0
.start_y
    dw $0120,$0120
init_platform_2_walk_routine:
    ldx $15E9
    stz !state,x
    lda !direction,x
    asl
    phx
    tax
    rep #$20
    lda .start_x,x
    sta $00
    lda .start_y,x
    plx
    sep #$20
    sta !sprite_y_lo,x
    xba
    sta !sprite_y_hi,x
    lda $00
    sta !sprite_x_lo,x
    lda $01
    sta !sprite_x_hi,x
    rts

.start_x
    dw $0050,$0020
.start_y
    dw $0120,$0120
init_platform_3_walk_routine:
    ldx $15E9
    stz !state,x
    lda !direction,x
    asl
    phx
    tax
    rep #$20
    lda .start_x,x
    sta $00
    lda .start_y,x
    plx
    sep #$20
    sta !sprite_y_lo,x
    xba
    sta !sprite_y_hi,x
    lda $00
    sta !sprite_x_lo,x
    lda $01
    sta !sprite_x_hi,x
    rts

.start_x
    dw $0098,$0068
.start_y
    dw $00D0,$00D0

movement_routines:
    dw bush_1_walk_routine
    dw bush_2_walk_routine
    dw platform_1_walk_routine
    dw platform_2_walk_routine
    dw platform_3_walk_routine


bush_1_walk_routine:
    ldx $15E9
    lda !state,x
    beq .init
    cmp #$01
    beq .rise
    cmp #$02
    beq .walk
    cmp #$03
    beq .lower
    .init
        inc !state,x
        lda .rise_duration
        sta !countdown_timer,x
    .rise
        lda .rise_speed
        sta !y_speed,x
        stz !x_speed,x
        lda !countdown_timer,x
        bne +
            inc !state,x
            lda .duration
            sta !countdown_timer,x
        +
        rts

    .walk
        lda !direction,x
        phx
        tax
        lda .speed,x
        plx
        sta !x_speed,x
        stz !y_speed,x
        jsr bounce_while_walk
        lda !countdown_timer,x
        bne +
            inc !state,x
            lda .lower_duration
            sta !countdown_timer,x
        +
        rts
    .lower
        lda .lower_speed
        sta !y_speed,x
        stz !x_speed,x
        lda !countdown_timer,x
        bne +
            stz !subsequent_hits_counter
            stz !sprite_status,x
        +
        rts
.speed
    db $F8,$08
.duration
    db $60
.rise_duration
    db $20
.lower_duration
    db $20
.rise_speed
    db $F0
.lower_speed
    db $10


bush_2_walk_routine:
    ldx $15E9
    lda !state,x
    beq .init
    cmp #$01
    beq .rise
    cmp #$02
    beq .walk
    cmp #$03
    beq .lower
    .init
        inc !state,x
        lda .rise_duration
        sta !countdown_timer,x
    .rise
        lda .rise_speed
        sta !y_speed,x
        stz !x_speed,x
        lda !countdown_timer,x
        bne +
            inc !state,x
            lda .duration
            sta !countdown_timer,x
        +
        rts

    .walk
        lda !direction,x
        phx
        tax
        lda .speed,x
        plx
        sta !x_speed,x
        stz !y_speed,x
        jsr bounce_while_walk
        lda !countdown_timer,x
        bne +
            inc !state,x
            lda .lower_duration
            sta !countdown_timer,x
        +
        rts
    .lower
        lda .lower_speed
        sta !y_speed,x
        stz !x_speed,x
        lda !countdown_timer,x
        bne +
            stz !subsequent_hits_counter
            stz !sprite_status,x
        +
        rts
.speed
    db $F8,$08
.duration
    db $60
.rise_duration
    db $20
.lower_duration
    db $20
.rise_speed
    db $F0
.lower_speed
    db $10


platform_1_walk_routine:
    ldx $15E9
    lda !state,x
    beq .init
    cmp #$01
    beq .rise
    cmp #$02
    beq .walk
    cmp #$03
    beq .lower
    .init
        inc !state,x
        lda .rise_duration
        sta !countdown_timer,x
    .rise
        lda .rise_speed
        sta !y_speed,x
        stz !x_speed,x
        lda !countdown_timer,x
        bne +
            inc !state,x
            lda .duration
            sta !countdown_timer,x
        +
        rts

    .walk
        lda !direction,x
        phx
        tax
        lda .speed,x
        plx
        sta !x_speed,x
        stz !y_speed,x
        jsr bounce_while_walk
        lda !countdown_timer,x
        bne +
            inc !state,x
            lda .lower_duration
            sta !countdown_timer,x
        +
        rts
    .lower
        lda .lower_speed
        sta !y_speed,x
        stz !x_speed,x
        lda !countdown_timer,x
        bne +
            stz !subsequent_hits_counter
            stz !sprite_status,x
        +
        rts
.speed
    db $F8,$08
.duration
    db $60
.rise_duration
    db $20
.lower_duration
    db $20
.rise_speed
    db $F0
.lower_speed
    db $10


platform_2_walk_routine:
    ldx $15E9
    lda !state,x
    beq .init
    cmp #$01
    beq .rise
    cmp #$02
    beq .walk
    cmp #$03
    beq .lower
    .init
        inc !state,x
        lda .rise_duration
        sta !countdown_timer,x
    .rise
        lda .rise_speed
        sta !y_speed,x
        stz !x_speed,x
        lda !countdown_timer,x
        bne +
            inc !state,x
            lda .duration
            sta !countdown_timer,x
        +
        rts

    .walk
        lda !direction,x
        phx
        tax
        lda .speed,x
        plx
        sta !x_speed,x
        stz !y_speed,x
        jsr bounce_while_walk
        lda !countdown_timer,x
        bne +
            inc !state,x
            lda .lower_duration
            sta !countdown_timer,x
        +
        rts
    .lower
        lda .lower_speed
        sta !y_speed,x
        stz !x_speed,x
        lda !countdown_timer,x
        bne +
            stz !subsequent_hits_counter
            stz !sprite_status,x
        +
        rts
.speed
    db $F8,$08
.duration
    db $60
.rise_duration
    db $20
.lower_duration
    db $20
.rise_speed
    db $F0
.lower_speed
    db $10


platform_3_walk_routine:
    ldx $15E9
    lda !state,x
    beq .init
    cmp #$01
    beq .rise
    cmp #$02
    beq .walk
    cmp #$03
    beq .lower
    .init
        inc !state,x
        lda .rise_duration
        sta !countdown_timer
    .rise
        lda .rise_speed
        sta !y_speed,x
        stz !x_speed,x
        lda !countdown_timer,x
        bne +
            inc !state,x
            lda .duration
            sta !countdown_timer,x
        +
        rts

    .walk
        lda !direction,x
        phx
        tax
        lda .speed,x
        plx
        sta !x_speed,x
        stz !y_speed,x
        jsr bounce_while_walk
        lda !countdown_timer,x
        bne +
            inc !state,x
            lda .lower_duration
            sta !countdown_timer,x
        +
        rts
    .lower
        lda .lower_speed
        sta !y_speed,x
        stz !x_speed,x
        lda !countdown_timer,x
        bne +
            stz !subsequent_hits_counter
            stz !sprite_status,x
        +
        rts
.speed
    db $F8,$08
.duration
    db $60
.rise_duration
    db $20
.lower_duration
    db $20
.rise_speed
    db $F0
.lower_speed
    db $10

bounce_while_walk:
    rts
                    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GRAPHICS ROUTINE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

TileNums:
;Koopa
db $01,$01,$01,$01
;Goomba
db $00,$00,$00,$00
;Shelless Koopa
db $00,$00,$00,$00

TileProps:
;Koopas
db %00000101,%00000111,%00001001,%00001011
;Goombas
db %00000101,%00000111,%00001001,%00001011
;Shelless Koopas
db %00000101,%00000111,%00001001,%00001011

koopa_tiles:
db $20, $00

goomba_tiles:
db $02

shelless_koopa_tiles:
db $04

TilePointers:
dw koopa_tiles
dw koopa_tiles
dw koopa_tiles
dw koopa_tiles

dw goomba_tiles
dw goomba_tiles
dw goomba_tiles
dw goomba_tiles

dw shelless_koopa_tiles
dw shelless_koopa_tiles
dw shelless_koopa_tiles
dw shelless_koopa_tiles

EnemyXOff:
db $F8,$F8

EnemyYOff:
db $FE,$EE

!board_tile_count = #$05

target_board_tile_size:
    db $02,$02,$02,$02
    ;stick
    db $00,$00

target_board_tiles:
    db $06,$06,$06,$06
    ;stick
    db $08,$08

target_board_props:
    db %00001101, %01001101, %10001101, %11001101
    ;stick
    db %00000001, %00000001
target_board_x_off:
    db $F0,$00,$F0,$00
    ;stick
    db $FC,$FC
target_board_y_off:
    db $F0,$F0,$00,$00
    ;stick
    db $10,$18

SUB_GFX:            jsr GetDrawInfo         ;Get all the drawing info, duh
                    stz $07

                    jsr draw_enemy
                    jsr draw_board

                    LDA $07
                    LDY #$FF
                    JSL $01B7B3                 ;/and then draw em
                    EndIt:
                    RTS
                    

draw_enemy:
    PHX
    lda !direction,x
    and #$01
    asl #6
    sta $04
    LDA !enemy_type,x
    sta $03
    asl
    TAX
    REP #$20
    LDA TilePointers,x
    sta $05
    sep #$20

    txa
    lsr
    tax
    lda TileProps,x
    sta $08

    lda TileNums,x
    tax
    .GFXLoop
    
    LDA $00
    CLC
    ADC EnemyXOff,x
    STA $0300,y
    
    LDA $01
    CLC
    ADC EnemyYOff,x
    STA $0301,y
    
    PHY
    TXY
    LDA ($05),y
    PLY
    STA $0302,y
    PHX
    LDX $15E9
    LDA $15F6,x
    PLX
    ORA $64
    ORA $08
    ora $04
    STA $0303,y
    PHY
    TYA
    LSR #2
    TAY
    LDA #$02
    PHX
    LDX $15E9
    ORA $15A0,x ; horizontal offscreen flag
    PLX
    STA $0460,y ;
    PLY
    INY
    INY
    INY
    INY
    inc $07
    DEX
    BPL .GFXLoop
    PLX
    rts

draw_board:
    phx
    lda !board_tile_count
    tax
    .GFXLoop
    
    LDA $00
    CLC
    ADC target_board_x_off,x
    STA $0300,y
    
    LDA $01
    CLC
    ADC target_board_y_off,x
    STA $0301,y
    
    lda target_board_tiles,x
    STA $0302,y

    PHX
    LDX $15E9
    LDA $15F6,x
    PLX
    ORA $64
    ORA target_board_props,x
    STA $0303,y
    PHY
    TYA
    LSR #2
    TAY
    LDA target_board_tile_size,x
    PHX
    LDX $15E9
    ORA $15A0,x ; horizontal offscreen flag
    PLX
    STA $0460,y ;
    PLY
    INY
    INY
    INY
    INY
    inc $07
    DEX
    BPL .GFXLoop
    PLX
    rts
                    
                
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;       Y = index to sprite OAM ($300)
;       $00 = sprite x position relative to screen boarder
;       $01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    
DATA_03B75C:        db $0C,$1C
DATA_03B75E:        db $01,$02

GetDrawInfo:        STZ $186C,X               ; Reset sprite offscreen flag, vertical 
                    STZ $15A0,X               ; Reset sprite offscreen flag, horizontal 
                    LDA $E4,X                 ; \ 
                    CMP $1A                   ;  | Set horizontal offscreen if necessary 
                    LDA $14E0,X               ;  | 
                    SBC $1B                   ;  | 
                    BEQ ADDR_03B774           ;  | 
                    INC $15A0,X               ; / 
ADDR_03B774:        LDA $14E0,X               ; \ 
                    XBA                       ;  | Mark sprite invalid if far enough off screen 
                    LDA $E4,X                 ;  | 
                    REP #$20                  ; Accum (16 bit) 
                    SEC                       ;  | 
                    SBC $1A                   ;  | 
                    CLC                       ;  | 
                    ADC.W #$0040              ;  | 
                    CMP.W #$0180              ;  | 
                    SEP #$20                  ; Accum (8 bit) 
                    ROL                       ;  | 
                    AND.B #$01                ;  | 
                    STA $15C4,X               ;  | 
                    BNE ADDR_03B7CF           ; /  
                    LDY.B #$00                ; \ set up loop: 
                    LDA $1662,X               ;  |  
                    AND.B #$20                ;  | if not smushed (1662 & 0x20), go through loop twice 
                    BEQ ADDR_03B79A           ;  | else, go through loop once 
                    INY                       ; /                        
ADDR_03B79A:        LDA $D8,X                 ; \                        
                    CLC                       ;  | set vertical offscree 
                    ADC DATA_03B75C,Y         ;  |                       
                    PHP                       ;  |                       
                    CMP $1C                   ;  | (vert screen boundry) 
                    ROL $00                   ;  |                       
                    PLP                       ;  |                       
                    LDA $14D4,X               ;  |                       
                    ADC.B #$00                ;  |                       
                    LSR $00                   ;  |                       
                    SBC $1D                   ;  |                       
                    BEQ ADDR_03B7BA           ;  |                       
                    LDA $186C,X               ;  | (vert offscreen)      
                    ORA DATA_03B75E,Y         ;  |                       
                    STA $186C,X               ;  |                       
ADDR_03B7BA:        DEY                       ;  |                       
                    BPL ADDR_03B79A           ; /                        
                    LDY $15EA,X               ; get offset to sprite OAM                           
                    LDA $E4,X                 ; \ 
                    SEC                       ;  |                                                     
                    SBC $1A                   ;  |                                                    
                    STA $00                   ; / $00 = sprite x position relative to screen boarder 
                    LDA $D8,X                 ; \                                                     
                    SEC                       ;  |                                                     
                    SBC $1C                   ;  |                                                    
                    STA $01                   ; / $01 = sprite y position relative to screen boarder 
                    RTS                       ; Return 

ADDR_03B7CF:        PLA                       ; \ Return from *main gfx routine* subroutine... 
                    PLA                       ;  |    ...(not just this subroutine) 
                    RTS                       ; / 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
SPR_T12:             db $40,$B0
SPR_T13:             db $01,$FF
SPR_T14:             db $30,$C0,$A0,$C0,$A0,$F0,$60,$90     ;bank 1 sizes
                    db $30,$C0,$A0,$80,$A0,$40,$60,$B0      ;bank 3 sizes
SPR_T15:             db $01,$FF,$01,$FF,$01,$FF,$01,$FF     ;bank 1 sizes
                    db $01,$FF,$01,$FF,$01,$00,$01,$FF      ;bank 3 sizes

SUB_OFF_SCREEN_X1:   LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SUB_OFF_SCREEN_X2:   LDA #$04                ;  | 
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X3:   LDA #$06                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X4:   LDA #$08                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X5:   LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X6:  LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X7:   LDA #$0E                ;  |
STORE_03:           STA $03                 ;  |            
                    BRA START_SUB           ;  |
SUB_OFF_SCREEN_X0:   STZ $03                    ; /

START_SUB:           JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ; | 
                    ADC #$50                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; | 
                    CMP #$02                ; | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13
                    AND #$01
                    ORA $03
                    STA $01
                    TAY
                    LDA $1A
                    CLC
                    ADC SPR_T14,y
                    ROL $00
                    CMP $E4,x
                    PHP
                    LDA $1B
                    LSR $00
                    ADC SPR_T15,y
                    PLP
                    SBC $14E0,x
                    STA $00
                    LSR $01
                    BCC SPR_L31
                    EOR #$80
                    STA $00
SPR_L31:             LDA $00
                    BPL RETURN_35
ERASE_SPRITE:        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /    
                    LDY $161A,x
                    CPY #$FF
                    BEQ KILL_SPRITE
                    LDA #$00
                    STA $1938,y
KILL_SPRITE:         STZ $14C8,x             ; erase sprite
RETURN_35:           RTS                     ; return

VERTICAL_LEVEL:      LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; | 
                    BCS RETURN_35           ; /
                    LDA $E4,x               ; \ 
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13
                    LSR A
                    AND #$01
                    STA $01
                    TAY
                    LDA $1C
                    CLC
                    ADC SPR_T12,y
                    ROL $00
                    CMP $D8,x
                    PHP
                    LDA.w $001D 
                    LSR $00
                    ADC SPR_T13,y
                    PLP
                    SBC $14D4,x
                    STA $00
                    LDY $01
                    BEQ SPR_L38
                    EOR #$80
                    STA $00
SPR_L38:             
                    LDA $00
                    BPL RETURN_35
                    BMI ERASE_SPRITE

SUB_IS_OFF_SCREEN:   LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ; |  
                    RTS                     ; / return

