incsrc "sublib_defines.asm"

; misc settings
!tile	= $00
!xdisp	= $03
!ydisp	= $04

!slowdown_frames = $1E

; ram
!game_status = $0F42


; specific sprite tables
!slowdown_timer = $1540

;----------------------------------------

print "INIT ",pc
		LDA #$00
		STA !spr_cust_spr_clipping,x

		LDA #!slowdown_frames
		STA !slowdown_timer,x
		RTL

print "MAIN ",pc
		PHB
		PHK
		PLB
		JSR main
		PLB
		RTL
		
;----------------------------------------
	
main:
		JSR draw_gfx
		LDA !spr_status,x
		EOR #$08
		ORA $9D
		BNE .return
		
		JSL !off_screen
		
		; update x/y positions w/o gravity
		JSL $818022
		JSL $81801A
		; gravity
		; JSL $81802A
		
		; interact with blocks
		JSL $819138
		
		JSR sides_collision
		JSR react_to_collisions
		JSR handle_slowdown
		JSR check_if_stopped
		
	.return:
		RTS
		
;----------------------------------------
		
sides_collision:
		LDA !spr_xpos_lo,x
		CMP #$08
		BCS .check_right_side
		LDA #$08
		STA !spr_xpos_lo,x
		LDA #$02
		STA $1588,x
		RTS
		
	.check_right_side:
		CMP #$E8
		BCC .return
		LDA #$E8
		STA !spr_xpos_lo,x
		LDA #$01
		STA $1588,x
	.return:
		RTS
	
	
	
react_to_collisions:
		; udlr
		LDA !spr_block_collision,x
		AND #$0C
		BEQ .check_horiz
		
		; collided horizontally
		LDA !spr_yspeed,x
		EOR #$FF
		INC
		BPL +
		INC
		INC
		BRA ++
	+
		DEC
		DEC
	++
		STA !spr_yspeed,x
		; fixes bouncing off of shattering blocks
		RTS
		
	.check_horiz:
		LDA !spr_block_collision,x
		AND #$03
		BEQ .no_horiz
		
		; collided_vertically
		LDA !spr_xspeed,x
		EOR #$FF
		INC
		BPL +
		INC
		INC
		BRA ++
	+
		DEC
		DEC
	++
		STA !spr_xspeed,x
		
	.no_horiz:
		RTS
		
		
		
handle_slowdown:
		LDA !slowdown_timer,x
		BNE .return
		
		LDA #!slowdown_frames
		STA !slowdown_timer,x
		
		LDA !spr_xspeed,x
		BEQ .check_y
		BMI +
		DEC !spr_xspeed,x
		BRA .check_y
	+
		INC !spr_xspeed,x
		
	.check_y:
		LDA !spr_yspeed,x
		BEQ .return
		BMI +
		DEC !spr_yspeed,x
		BRA .return
	+
		INC !spr_yspeed,x
		
	.return:
		RTS
		
		
		
check_if_stopped:
		LDA !spr_xspeed,x
		BNE .not_dead
		LDA !spr_yspeed,x
		BNE .not_dead
		
		INC !game_status
		
		LDA #$04
		STA !spr_status,x
		JSL $87FC3B			; fancy spin jump effects
		LDA #$08
		STA $1DF9
		
	.not_dead:
		RTS
		
		
		
;----------------------------------------

draw_gfx:
		JSL !get_draw_info
		
		LDA $00
		SEC
		SBC #!xdisp
		STA $0300,y
		LDA $01
		SEC
		SBC #!ydisp
		STA $0301,y
		LDA #!tile
		STA $0302,y
		LDA !spr_tile_props,x
		ORA $64
		STA $0303,y
		
		; one 16x16 tile
		LDY #$02
		LDA #$00
		JSL $81B7B3
		RTS
		