incsrc "sublib_defines.asm"

; misc settings
!tile = $06

!ball_tile = $00
!ball_props = $09

!ball_sprn = $01

!ball_xdisp = $01
!ball_ydisp = $00

!ball_spawn_xdisp = $04
!ball_spawn_ydisp = $04

!max_angle = $74
!min_angle = $0C
!radius = $20
!speed_radius = $38

!aiming_speed = $02

!aim_sound_n = $01
!aim_sound_c = $1DFC
!shoot_sound_n = $06
!shoot_sound_c = $1DFC

; ram
!game_status = $0F42

; specific sprite tables
!curr_angle				= !spr_misc1
!curr_angle_dir			= !spr_misc2	; 0 right, 1 left
!center_x_lo			= !spr_misc3
!center_y_lo			= !spr_misc4
!center_x_hi			= !spr_misc5
!center_y_hi			= !spr_misc6

;----------------------------------------

print "INIT ",pc
		LDA #!max_angle
		STA !curr_angle,x
		
		LDA !spr_xpos_lo,x
		STA !center_x_lo,x
		LDA !spr_ypos_lo,x
		STA !center_y_lo,x
		LDA !spr_xpos_hi,x
		STA !center_x_hi,x
		LDA !spr_ypos_hi,x
		STA !center_y_hi,x
		RTL
		

print "MAIN ",pc
		PHB
		PHK
		PLB
		JSR main
		PLB
		RTL
		
;----------------------------------------
	
main:
		JSR draw_gfx
		LDA !spr_status,x
		EOR #$08
		ORA $9D
		BNE .return
		
		JSR update_position
		JSR next_angle
		JSR flash_color
		
		JSR check_shoot
		
	.return:
		RTS
		
;----------------------------------------

update_position:
		STA $666666

		LDA !curr_angle,x
		REP #$20
		AND #$00FF
		JSL !get_sin
		SEP #$20
		STA $211B
		XBA
		STA $211B
		LDA #!radius
		STA $211C
		LDA !center_y_lo,x
		SEC
		SBC $2135
		STA !spr_ypos_lo,x
		; LDA !center_y_hi,x
		; SBC #$00
		; STA !spr_ypos_hi,x
		
		LDA !curr_angle,x
		REP #$20
		AND #$00FF
		JSL !get_cos
		SEP #$20
		STA $211B
		XBA
		STA $211B
		LDA #!radius
		STA $211C
		LDA !center_x_lo,x
		CLC
		ADC $2135
		STA !spr_xpos_lo,x
		; LDA !center_x_hi,x
		; ADC #$00
		; STA !spr_xpos_hi,x
		
		; update x/y positions w/o gravity
		JSL $818022
		JSL $81801A
		RTS
		
		
		
next_angle:
		LDA $14
		AND #$03
		BNE .skip_sound
		LDA #!aim_sound_n
		STA !aim_sound_c
	.skip_sound:
		
		LDA !curr_angle_dir,x
		BEQ .decrease
		
		LDA !curr_angle,x
		CLC
		ADC #$02
		STA !curr_angle,x
		CMP #!max_angle
		BNE .return
		STZ !curr_angle_dir,x
	.return:
		RTS
		
	.decrease:
		LDA !curr_angle,x
		SEC
		SBC #$02
		STA !curr_angle,x
		CMP #!min_angle
		BNE .return
		INC !curr_angle_dir,x
		RTS
		
		
		
flash_color:
		LDA $14
		AND #$03
		BNE .return
		
		LDA !spr_tile_props,x
		EOR #%00001100
		STA !spr_tile_props,x
	.return:
		RTS
		
		
		
check_shoot:
		; check for B press
		LDA $16
		BMI .shoot_ball
		RTS
		
	.shoot_ball:
		LDA #!shoot_sound_n
		STA !shoot_sound_c
		
		; find sprite slot (including A and B)
		JSL $82A9E4
		BPL .found_slot
		LDY #$0A				; unlikely but you know
	.found_slot:
		LDA #$01
		STA !spr_status,y
		PHX
		TYX
		LDA #!ball_sprn
		STA !spr_number_c,x
		PLX
		
		LDA !center_x_lo,x
		CLC
		ADC #!ball_spawn_xdisp
		STA.w !spr_xpos_lo,y
		LDA !center_x_hi,x
		ADC #$00
		STA !spr_xpos_hi,y
		
		LDA !center_y_lo,x
		CLC
		ADC #!ball_spawn_ydisp
		STA.w !spr_ypos_lo,y
		LDA !center_y_hi,x
		ADC #$00
		STA !spr_ypos_hi,y
		
		PHX
		TYX
		JSL $87F7D2
		JSL $8187A7
		LDA #$08
		STA !spr_flags_c,x
		PLX
		
		LDA !curr_angle,x
		REP #$20
		AND #$00FF
		JSL !get_sin
		SEP #$20
		STA $211B
		XBA
		STA $211B
		LDA #!speed_radius
		STA $211C
		LDA $2135
		EOR #$FF
		INC
		STA.w !spr_yspeed,y
		
		LDA !curr_angle,x
		REP #$20
		AND #$00FF
		JSL !get_cos
		SEP #$20
		STA $211B
		XBA
		STA $211B
		LDA #!speed_radius
		STA $211C
		LDA $2135
		; EOR #$FF
		; INC
		STA.w !spr_xspeed,y
		
		INC !game_status
		
		; rip
		STZ !spr_status,x
		RTS
		
		
		
;----------------------------------------

draw_gfx:
		JSL !get_draw_info
		
		; get ball position
		LDA !center_x_lo,x
		SEC
		SBC $1A
		STA $02
		LDA !center_y_lo,x
		SEC
		SBC $1C
		STA $03
		
		LDA $00
		STA $0300,y
		LDA $01
		STA $0301,y
		LDA #!tile
		STA $0302,y
		LDA !spr_tile_props,x
		ORA $64
		STA $0303,y
		
		LDA $02
		CLC
		ADC #!ball_xdisp
		STA $0304,y
		LDA $03
		CLC
		ADC #!ball_ydisp
		STA $0305,y
		LDA #!ball_tile
		STA $0306,y
		LDA !spr_tile_props,x
		ORA #$02
		ORA $64
		STA $0307,y
		
		; two 16x16 tiles
		LDY #$02
		LDA #$01
		JSL $81B7B3
		RTS
		

