prot Mode7Data;,Mode7Pal
table "ascii-28.txt"

;; TODO:
;;  - Change how acceleration works (Make it accelerate correctly rather than setting the speed based on rotation)
;;  - Optimize. (lol nope)

!asm	= lives_asm	; > ASM folder (lives_asm = Uses live system, timed_asm = Uses a timer system)
!cap	= $20	; > How far the platform can rotate
!timer	= $60	; > Timer (16-bit)
!bonusState	= $62	; > What state the bonus game is in (00 - Instructions, 01 - Bonus Game, 02 - Result Screen)
!missed		= $63	; > Counter. Keeps track of how many items the player missed

!lives		= $05	; > How many items you are allowed to miss. (Only 0-9 since I'm too lazy to handle hex -> dec conversion lol)
!resTime	= $00C0	; > How long the result screen will stay
!pltfrmLnth2	= $30	; > Distance between the center and the edge of the platform (When platform is small)
!pltfrmLnth		= $40	; > Distance between the center and the edge of the platform (When platform is normal)
!pltfrmLnth1	= $50	; > Distance between the center and the edge of the platform (When platform is big)
!pltfrmRAM	= $64	; > ram that holds how big the platform currently is
!speedUp	= $0400	; > Time when this speed up. Note: When using timed_asm this will act as the 
!spdUpTxt	= $5C	; > Display Speed up message
!page	= $58	; > ram that stores what page the player is on.

!rtrnLvl	= $0105	; > Level to return to (Format: ----WPSx xxxxxxxx)
; xxxxxxxxx: What exit to go to (0-1FF)
; S = Secondary exit flag
; P = Exit present flag (Will be set automatically)
; W = If bit S is set this is the Water level flag. Otherwise this is the "go to midway" flag.
; - = Unused. Won't affect anything to my knowledge
; Note: Setting this to 0000 will warp to the overworld.

print "------- General  Info -------"
print "- Inserting !asm"

incsrc !asm/def/text.asm	; > Text defines
incsrc !asm/def/player.asm	; > player defines
incsrc !asm/def/sprites.asm	; > Sprite defines
incsrc !asm/def/mode7.asm	; > Mode7 defines

incsrc !asm/text.asm	; > Routines that handle drawing the text on screen. Also holds the text itself.
incsrc !asm/player.asm	; > Handles player position and state. Also handles sprite spawning.
incsrc !asm/sprites.asm	; > Handles sprite position and player interaction.
incsrc !asm/mode7.asm	; > Holds the Mode7 data. Also handles uploading GFX and updating HDMA tables.

load:
	REP #$20
		LDA #$0100	; \ Set mode 7 stuff up
		STA $2E		; |
		STA $34		; |
		TDC			; |
		STA $30		; |
		STA $32		; /
	SEP #$20
	RTL

init:
	LDX #$80	; \ Fix OAM... Dunno must've screwed up setting the tile size somewhere but this fixes it so idc lol.
-	STZ $0420,x	; |
	DEX : BPL -	; /
	LDA #$10	; \ Move sprites to subscreen
	STA $212d	; /
	LDA #$03	; \ Move layer 1 to mainscreen
	STA $212c	; /
	LDA #$0B	; \ Freeze player
	STA $71		; /
	LDA #$C3	; \ Activate bonus mode
	STA $0D9B	; /
	JSR UploadGFX	; > Upload graphics
	JSR InitSpriteTables
	REP #$20
		STZ !timer	; > Initialize timer
		LDA #$2020	; \ Initial scaling
		STA $38		; /
		LDA #$0040	; \ Center Y position - $80
		STA $2C		; /
		LDA $1A		; \ Initial position
		STA $3A		; |
		LDA $1C		; |
		STA $3C		; /
		LDA #$0000	; \ Initial rotation
		STA $36		; /
		STA $2A		; > Center X position - $80
	SEP #$20
	LDA $2A
	CLC : ADC #$77
	STA !plyrX
	LDA $2C
	CLC : ADC #$6F
	STA !plyrY
	STZ !bonusState
	STZ !missed
	STZ !page
	STZ !spdUpTxt
	JSR SetupHDMA
	LDA #!pltfrmLnth
	STA !pltfrmRAM
	LDA $19
	STA $0DA1
	STZ $19
	RTL

main:
	STZ $7D
	LDA #$38	; Enable HDMA on channel 3, 4 & 5
	TSB $420C	;
	LDA #$FF : STA $78	; > Hide player
	LDA #$05	; \ Prevent timer from decrementing during the bonus game
	STA $0F30	; /
	LDA !bonusState
	ASL
	TAX
	JMP.w (.pntr,x)

.pntr
	dw DispInstructions
	dw RunBonusGame
	dw DispResults

DispInstructions:
	LDA $16			; \ Check if a left or right is pressed
	AND #$03		; |
	BEQ .noButton	; /
	AND #$01		; \ Store 1 if right is pressed. 0 if left is pressed
	STA !page
.noButton
	LDA !page
	JSR DrawText	; > Draw info text
	JSR PlayerGFX	; > Draw the player
	LDA $16		; \ Wait until player presses B button
	AND #$80	; |
	BEQ .return	; /
	INC !bonusState	; > Next bonus state (Run actual game)
.return
	JSL $01ACF9	; > Randomize value to get "more random" results when starting a new bonus game
	RTL
	
DispResults:
	JSR Mode7HDMA	; > Update HDMA table
	JSR DrawGameOver
	REP #$20
		INC !timer		; > Increase timer
		LDA !timer		; \ Check if it's time to return to level
		CMP.w #!resTime	; |
		BNE .return		; /
		LDA.w #!rtrnLvl	; \ Send player out of bonus game
		BEQ .OW			; /
		ORA #$0400		; > If not going to overworld add "exit present" flag
	SEP #$20
	STA $19B8	; > Store level to return to
	XBA
	STA $19D8	; > Store highbit and exit settings
	LDA #$05	; \ Teleport player
	STA $71		; /
	LDA $0DA1
	STA $19
	RTL	; > Return
.return
	SEP #$20
	RTL
.OW	SEP #$20
	LDA #$0B	; \ Set gamemode to 0B (Fade to overworld)
	STA $0100	; /
	LDA $0DA1
	STA $19
	RTL	; > Return

RunBonusGame:
	REP #$20
	INC !timer
	LDA $15			; \ Check direction the player presses and update platform accordingly
	AND #$0003		; |
	BEQ .noChange	; |
	AND #$0001		; |
	BEQ .left		; |
	LDA $36			; |
	INC				; |
	CMP.w #1+!cap	; |
	BEQ .noChange	; |
	STA $36			; |
	BRA .noChange	; |
.left				; |
	LDA $36			; |
	DEC				; |
	CMP.w #$FFFF-!cap;|
	BEQ .noChange	; |
	STA $36			; |
.noChange			; /
	SEP #$20
	JSR Player			; > Update player
	JSR DoSpriteStuff	; > Update sprites
	JSR DrawLives		; > Draw how many "lives" the player has left
	JSR Mode7HDMA
	RTL