;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; -- Player routines -- ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;

AccelSpeed:
	db $00,$10,$10,$10,$18,$18,$18,$20,$20,$20,$28,$28,$28,$30,$30,$30,$30
YSpeed:
	db $00,$00,$00,$00,$05,$05,$05,$05,$0A,$0A,$0A,$0A,$0F,$0F,$0F,$0F,$10

FallingOff:
	LDX #$02		; \ Update X position (directly rather than updating !plyrDIST then converting it to !plyrX)
	JSR UpdateYX	; /
	JSR UpdateYGravity	; > Update Y position with gravity
	LDA !plyrY
	CMP #$F0
	BCC	+
	INC !plyrSTATE	; > Next state (02 -- Inactive)
+	JSR PlayerGFX
	RTS

Player:
	LDA !plyrSTATE	; \ Check player state
	CMP #$01		; |
	BEQ FallingOff	; |
	CMP #$02		; |
	BEQ GameOver	; /
	REP #$20
		LDA $36	; \ Use	Mode 7 rotation as angle
		STA $04	; /
		JSR MaybeSpawnASprite	; > Returns with 8-bit A
	LDA $04
	PHP
	BPL +
	EOR #$FF
	INC
+	LSR : TAX
	LDA YSpeed,x
	STA !plyrYSpeed
	LDA AccelSpeed,x	; > Load speed values
	PLP					; \ Invert speed if necessary
	BPL +				; |
	EOR #$FF			; |
	INC					; /
+	STA !plyrXSpeed		; > Store new X speed of player
	STA !plyrXTmp		; > Store new X speed of player
	LDX #$01		; \ Update player X position
	JSR UpdateYX	; /
	LDA !plyrDIST	; \ Set distance from center position
	STA $06			; /
	JSR GetPosOffset	; > Calculate position
;	JSR UpdateMode7Reg	; > Restore the mode 7 registers that were used by the routine above
	LDA $2A			; \ Load mode 7 center X position (low)
	CLC : ADC $07	; | Add x offset
	CLC : ADC #$77	; | Add missing offset
	STA !plyrX		; / Store as player X
	LDA $2C			; \ Load mode 7 center Y position (low)
	CLC : ADC $08	; | Add y offset
	CLC : ADC #$6F	; | Add missing offset
	STA !plyrY		; / Store as player Y
	JSR PlayerGFX	; > Draw the player
.return
	RTS
	
GameOver:
	LDA #$14		; \ Play "bonus done" sound
	STA $1DFB		; /
	INC !bonusState	; > Next bonus state (02 -- Display result)
;	LDA #!lives		; \ Set lives to 0
;	STA !missed		; /
	REP #$20
		STZ $36		; > Reset rotation
		STZ !timer	; > Reset timer
	SEP #$20
	
	;; Switch, blue
	
	LDA #$01	; 1 activate, 0 deactivate
	STA $1F29	; blue
	RTS

PlayerGFX:
	LDA !plyrX
	STA.w !plyrOAM
	LDA !plyrY
	STA.w !plyrOAM+1
	LDA #!plyrTILE
	STA.w !plyrOAM+2
	LDA #!plyrPROP
	STA.w !plyrOAM+3
	LDA #$02
	STA !plyrSZE
	RTS

; Input:
;  X = 0 > Update Y position
;  X = 1 > Update X position (plyrDIST)
;  X = 2 > Update X position (plyrX)
UpdateYX:
	LDA !plyrSTATE
	CMP #$01
	BEQ ++
	LDA !plyrDIST		; \ Check if sprite is isn't on platform anymore
	BPL +				; |
	EOR #$FF			; |
	INC					; |
+	CMP !pltfrmRAM		; |
	BCC ++				; |
	INC !plyrSTATE		; | Next state if sprites isn't on platform anymore (01 -- Falling off of platform)
	BRA .return			; |
++	LDA !plyrYSpeed,x	; /
	BEQ .return				; If speed is 0, skip
	ASL                       ; \ Multiply speed by 16
	ASL                       ; | 
	ASL                       ; |
	ASL                       ; /
	CLC
	ADC.W $14EC
	STA.W $14EC
	PHP
	LDY.B #$00
	LDA !plyrYSpeed,x		; Load X speed 
	LSR                       ; \  
	LSR                       ; |Divide speed by 16 
	LSR                       ; | 
	LSR                       ; /  
	CMP.B #$08
	BCC +
	ORA.B #$F0
	DEY
+	PLP
	ADC !plyrY,x       ; \ Add value to X position 
	STA !plyrY,x       ; /
.return
	RTS				; Return

;; Taken from all.log
;; Modified to work with new addresses
UpdateYGravity:
	LDX #$00
	JSR UpdateYX  	 ; Update Y position
	LDY.B #$00			;
	LDA !plyrYSpeed	;\ 
	CLC				;| add 03 if sprite address is 00(case1), 01 if it's not (case2)
	ADC $019030		;|
	STA !plyrYSpeed	;/ to Ysp
	BMI .return		; if that makes the sprite go up, 
	CMP $01902E		;\ if speed is slower than 40 for case1, or 10 for case2, 
	BCC .return		;/
	LDA $01902E		;\ 
	STA !plyrYSpeed	;/ make that the speed
.return
	RTS

; Note: Called with 16-bit A. Returns with 8-bit A
MaybeSpawnASprite:
	LDA !timer
	CMP #!speedUp
	BCS +
	LDA #$003F
	BRA ++
+	JSR DrawSpeedUp
	LDA #$001F
++	SEP #$20
	AND $13			; \ Handle sprite spawning
	BNE .noSpawn	; |
	LDX #!sprAmount-1;|
-	LDA !state,x	; |
	BNE +			; |
	INC				; |
	STA !state,x	; |
	BRA .noSpawn	; |
+	DEX : BPL -		; /
.noSpawn
	RTS

;circle routine
;INPUT:
; $04 = angle (16bit)
;	you can make it go above $01FF, the routine will just clear the higher bits
; $06 = radius (8bit)
;OUTPUT:
; $07 = X displacement (add it to something like a center position, don't directly use it)
; $08 = Y displacement (same as above)

GetPosOffset:
; > Circle Y
	LDA $05 : LSR
	LDX $04
	LDA sincostable,x
	STA $211B
	STZ $211B
	LDA $06
	STA $211C
	LDA $2135
	BCC +
	EOR #$FF : INC A
+	STA $08
; > Circle X
	LDA $04
	CLC : ADC #$80
	TAX
	LDA $05
	ADC #$00 : LSR
	LDA sincostable,x
	STA $211B
	STZ $211B
	LDA $06
	STA $211C
	LDA $2135
	BCC +
	EOR #$FF : INC A
+	STA $07
RTS

sincostable:
db $00,$03,$06,$09,$0C,$0F,$12,$15,$19,$1C,$1F,$22,$25,$28,$2B,$2E
db $31,$35,$38,$3B,$3E,$41,$44,$47,$4A,$4D,$50,$53,$56,$59,$5C,$5F
db $61,$64,$67,$6A,$6D,$70,$73,$75,$78,$7B,$7E,$80,$83,$86,$88,$8B
db $8E,$90,$93,$95,$98,$9B,$9D,$9F,$A2,$A4,$A7,$A9,$AB,$AE,$B0,$B2
db $B5,$B7,$B9,$BB,$BD,$BF,$C1,$C3,$C5,$C7,$C9,$CB,$CD,$CF,$D1,$D3
db $D4,$D6,$D8,$D9,$DB,$DD,$DE,$E0,$E1,$E3,$E4,$E6,$E7,$E8,$EA,$EB
db $EC,$ED,$EE,$EF,$F1,$F2,$F3,$F4,$F4,$F5,$F6,$F7,$F8,$F9,$F9,$FA
db $FB,$FB,$FC,$FC,$FD,$FD,$FE,$FE,$FE,$FF,$FF,$FF,$FF,$FF,$FF,$FF

db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FE,$FE,$FE,$FD,$FD,$FC,$FC,$FB
db $FB,$FA,$F9,$F9,$F8,$F7,$F6,$F5,$F4,$F4,$F3,$F2,$F1,$EF,$EE,$ED
db $EC,$EB,$EA,$E8,$E7,$E6,$E4,$E3,$E1,$E0,$DE,$DD,$DB,$D9,$D8,$D6
db $D4,$D3,$D1,$CF,$CD,$CB,$C9,$C7,$C5,$C3,$C1,$BF,$BD,$BB,$B9,$B7
db $B5,$B2,$B0,$AE,$AB,$A9,$A7,$A4,$A2,$9F,$9D,$9B,$98,$95,$93,$90
db $8E,$8B,$88,$86,$83,$80,$7E,$7B,$78,$75,$73,$70,$6D,$6A,$67,$64
db $61,$5F,$5C,$59,$56,$53,$50,$4D,$4A,$47,$44,$41,$3E,$3B,$38,$35
db $31,$2E,$2B,$28,$25,$22,$1F,$1C,$19,$15,$12,$0F,$0C,$09,$06,$03
