;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; -- Sprite routines -- ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;

DoSpriteStuff:
	LDX #!sprAmount-1
-	LDA !state,x
	BEQ +
	JSR HandleSprite
+	DEX : BPL -
	RTS

Tiles:
	db $24,$24,$C2,$C2,$C2,$C2,$C2,$C2,$C2,$C2,$C2,$C2,$C2,$C2,$80,$82
Prop:
	db $0A,$0A,$04,$04,$04,$06,$06,$08,$04,$04,$04,$06,$06,$08,$0B,$09
Run:
	db $00,$00,$01,$01,$01,$02,$02,$03,$01,$01,$01,$02,$02,$03,$04,$05
; Run > What code to run. See below.

SprPntr:
	dw Up1		; 00
	dw Coins5	; 01
	dw Coins10	; 02
	dw Coins15	; 03
	dw PltfrmUp	; 04
	dw PltfrmDn	; 05

Up1:
	INC $18E4	; > Add live
	RTS
Coins5:
	LDA $13CC		; \ Add 5 to coins
	CLC : ADC #$05	; |
	STA $13CC		; /
	LDA #$01	; \ Play sound
	STA $1DFC	; /
	RTS
Coins10:
	LDA $13CC		; \ Add 10 to coins
	CLC : ADC #$0A	; |
	STA $13CC		; /
	LDA #$01	; \ Play sound
	STA $1DFC	; /
	RTS
Coins15:
	LDA $13CC		; \ Add 15 to coins
	CLC : ADC #$0F	; |
	STA $13CC		; /
	LDA #$01	; \ Play sound
	STA $1DFC	; /
	RTS
PltfrmUp:
	LDA #$29
	STA $1DFC
	LDA $3B
	CMP #$02
	BEQ .normal
	LDA #$01
	STA $3B
	STA $2B
	LDA #!pltfrmLnth1
	STA !pltfrmRAM
	RTS
.normal
	LDA #$00
	STA $3B
	STA $2B
	LDA #!pltfrmLnth
	STA !pltfrmRAM
	RTS
PltfrmDn:
	LDA #$29
	STA $1DFC
	LDA $3B
	CMP #$01
	BEQ PltfrmUp_normal
	LDA #$02
	STA $3B
	STA $2B
	LDA #!pltfrmLnth2
	STA !pltfrmRAM
	RTS

CheckContact:
	LDA !plyrX	; \ Player X pos low byte
	STA $00		; /
	STZ $08		; > Player X pos high byte
	LDA !plyrY	; \ Player Y pos low byte
	STA $01		; /
	STZ $09		; > Player Y pos high byte
	LDA #$0C	; \ Player width
	STA $02		; /
	STA $03		; > Player height
	STA $06		; > Sprite width
	STA $07		; > Sprite height

	LDA !X,x	; \ Sprite X pos low byte
	STA $04		; /
	STZ $0A		; > Sprite X pos high byte
	LDA !Y,x	; \ Sprite Y pos low byte
	STA $05		; /
	STZ $0B		; > Sprite Y pos high byte

	JSL $03B72B	; > Check for contact
	BCS .contact
	RTS
.contact
	LDA !sprTPE,x	; \ Load sprite type into X
	PHX : TAX		; /
	LDA Run,x : ASL : TAX	; > Get index for routine
	JSR (SprPntr,x)	; > Run routine
	PLX : PLA : PLA	; \ Kill sprite
	JMP KillSprite	; /

HandleSprite:
	CMP #$01
	BNE .noInit
	INC
	STA !state,x
	LDA !pltfrmRAM
	ASL
	JSR RANDOM
	CLC : ADC.b #$78
	SEC : SBC !pltfrmRAM
	STA !X,x
	LDA.b #Prop-Tiles-1
	JSR RANDOM
	STA !sprTPE,x
	LDA #$FF
	STA !Y,x
	LDA.b #!maxSpeed-!minSpeed
	JSR RANDOM
	CLC : ADC #!minSpeed
	STA !YSpeed,x
.noInit
	JSR CheckContact
	TXA : ASL : ASL : TAY
	LDA !X,x
	STA $0200,y
	LDA !Y,x
	STA $0201,y
	LDA !sprTPE,x
	PHX : TAX
	LDA Tiles,x
	STA $0202,y
	LDA Prop,x
	STA $0203,y
	PLX
	LDA #$02
	STA $0420,x
	JSR SPRupdateY
	RTS

SetMissed:
	PLX
	INC !missed
	LDA !missed
	CMP #!lives
	BNE KillSprite
	JMP GameOver
KillSprite:
	LDA #$00		; \ Zero out sprite tables for current sprite
	STA !state,x	; |
	STA !X,x		; |
	STA !Y,x		; |
	STA !XSpeed,x	; |
	STA !YSpeed,x	; /
	RTS

InitSpriteTables:
	LDA #$00
	LDX #!sprAmount-1	; \ Initialize all sprite tables
-	STA !state,x		; |
	STA !X,x			; |
	STA !Y,x			; |
	STA !XSpeed,x		; |
	STA !YSpeed,x		; |
	STA !sprTPE,x		; |
	STA !Xfrac,x		; |
	STA !Yfrac,x		; |
+	DEX : BPL -			; /
	STZ !plyrY			; \ Initialize player addresses
	STZ !plyrDIST		; |
	STZ !plyrX			; |
	STZ !plyrYSpeed		; |
	STZ !plyrXSpeed		; |
	STZ !plyrXTmp		; |
	STZ !plyrSTATE		; /
	RTS

; Input:
;   X = Slot
SPRupdateY:
	PHX : TXA
	CLC : ADC #!sprAmount
	TAX : JSR SPRupdateX
	CLC : ADC #$10
	BCS SetMissed
	PLX : RTS

; Input:
;   X = Slot
SPRupdateX:
	LDA !XSpeed,x
	BEQ .return				; If speed is 0, skip
	ASL                       ; \ Multiply speed by 16
	ASL                       ; | 
	ASL                       ; |
	ASL                       ; /
	CLC
	ADC !Xfrac,x
	STA !Xfrac,x
	PHP
	LDY.B #$00
	LDA !XSpeed,x		; Load X speed 
	LSR                       ; \  
	LSR                       ; |Divide speed by 16 
	LSR                       ; | 
	LSR                       ; /  
	CMP.B #$08
	BCC +
	ORA.B #$F0
	DEY
+	PLP
	ADC !X,x       ; \ Add value to X position
	STA !X,x       ; /
.return
	RTS				; Return

;============;
;RNG by 1024
;============;

RANDOM:
	PHX
	PHP
	SEP #$30
	PHA
	JSL $01ACF9
	PLX
	CPX #$FF
	BNE NORMALRT
	LDA $148B
	BRA ENDRANDOM
NORMALRT:
	INX
	LDA $148B
	STA $4202
	STX $4203
	NOP #4
	LDA $4217
ENDRANDOM:
	PLP
	PLX
	RTS