;;;;;;;;;;;;;;;;;;;;;;
;; -- HDMA stuff -- ;;
;;;;;;;;;;;;;;;;;;;;;;

SetupHDMA:
	LDX.b #.CD-.AB				; \ Load HDMA tables into ram so we can modify upper and lower half separately
-	LDA .AB,x					; |
	STA !HDMA,x					; |
	DEX : BPL -					; |
	LDX.b #SetupHDMA_end-.CD	; |
-	LDA .CD,x					; |
	STA !HDMA2,x				; |
	DEX : BPL -					; /
	REP #$20	; 16 bit A
	LDA #$1b03	; $43X0 = 00
	STA $4330	; $43x1 = 00
	LDA #$1d03
	STA $4340
	LDA.w #!HDMA	; get pointer to brightness table
	STA $4332	; store it to low and high byte pointer
	LDA.w #!HDMA2
	STA $4342
	LDY #$7F
	STY $4334	; store to bank pointer byte
	STY $4344	; store to bank pointer byte
	LDA #$3200		; | Use Mode 0 on register 2132
	STA $4350		; | 4350 = Mode, 4351 = Register
	LDA #.HDMA_Table	; | Address of HDMA table
	STA $4352		; | 4352 = Low-Byte of table, 4353 = High-Byte of table
	LDY.b #.HDMA_Table>>16	; | Address of HDMA table, get bank byte
	STY $4354		; | 4354 = Bank-Byte of table
	SEP #$20		;/
	LDA #%00100000
	TSB $0D9F
	SEP #$20	; 8 bit A
	RTS		; return

; HDMA tables.
; Only the bottom half is ever modified
.AB
	db $7F
	dw $0100,$0000	; > 16-bit parameter A,B (Top half)
	db $7F
	dw $0100,$0000	; > 16-bit parameter A,B (Bottom half)
	db $00
.CD
	db $7F
	dw $0000,$0100	; > 16-bit parameter C,D (Top half)
	db $7F
	dw $0000,$0100	; > 16-bit parameter C,D (Bottom half)
	db $00
.end

.HDMA_Table
	db $07,$60
	db $07,$61
	db $07,$62
	db $07,$63
	db $07,$64
	db $07,$65
	db $07,$66
	db $07,$67
	db $07,$68
	db $07,$69
	db $07,$6A
	db $07,$6B
	db $07,$6C
	db $07,$6D
	db $07,$6E
	db $07,$6F
	db $07,$70
	db $07,$71
	db $07,$72
	db $07,$73
	db $07,$74
	db $07,$75
	db $07,$76
	db $07,$77
	db $07,$78
	db $07,$79
	db $07,$7A
	db $07,$7B
	db $07,$7C
	db $07,$7D
	db $07,$7E
	db $07,$7F
	db $00

Mode7HDMA:
	PHP
	REP #$20
		LDA $2E	; \ Update HDMA table.
		STA !A	; |
		LDA $30	; |
		STA !B	; |
		LDA $32	; |
		STA !C	; |
		LDA $34	; |
		STA !D	; /
	PLP
RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; --          Mode  7         -- ;;
;; -- Upload routines and data -- ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

UploadGFX:
	LDA #$80            ; \ Increase on $2119 write.
	STA $2115           ; /
	REP #$10
	LDX #$0000
	STX $2116
	LDA #$01            ; \ Set mode to...
	STA $4300           ; / 2 regs write once
	LDA #$18            ; \ 
	STA $4301           ; / writing to $2118 AND $2119
	LDX.w #Mode7Data	; \ 
	STX $4302			; | 
	LDA.b #Mode7Data>>16; | bank where our data is
	STA $4304           ; /
	LDX #Mode7Data_end-Mode7Data          ;\ size of our data
	STX $4305           ;/
	SEP #$10
	LDA.b #$01	   ; \ start DMA
	STA.w $420B	   ; / transfer
	RTS

pushpc
freedata
Mode7Data:
	incbin gfx/platform.bin
.end
pullpc