;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; -- Sprite routines -- ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;

DoSpriteStuff:
	LDX #!sprAmount-1
-	LDA !state,x
	BEQ +
	JSR HandleSprite
+	DEX : BPL -
	RTS

Tiles:
	db $24,$C2,$C2,$C2,$C2,$C2,$C2,$C2,$C2,$C2,$C2,$C2,$C2,$C2,$80,$82
Prop:                     
	db $0A,$04,$04,$04,$04,$04,$06,$08,$04,$04,$04,$06,$06,$08,$0B,$09
Run:                      
	db $00,$01,$01,$01,$01,$01,$02,$03,$01,$01,$01,$02,$02,$03,$04,$05
; Run > What code to run. See below.

SprPntr:
	dw Up1		; 00
	dw Coins5	; 01
	dw Coins10	; 02
	dw Coins15	; 03
	dw PltfrmUp	; 04
	dw PltfrmDn	; 05

Up1:
	INC $18E4	; > Add live
	RTS
Coins5:
	LDA $13CC		; \ Add 5 coins
	CLC : ADC #$05	; |
	STA $13CC		; /
	LDA #$01	; \ Play sound
	STA $1DFC	; /
	RTS
Coins10:
	LDA $13CC		; \ Add 10 coins
	CLC : ADC #$0A	; |
	STA $13CC		; /
	LDA #$01	; \ Play sound
	STA $1DFC	; /
	RTS
Coins15:
	LDA $13CC		; \ Add 15 coins
	CLC : ADC #$0F	; |
	STA $13CC		; /
	LDA #$01	; \ Play sound
	STA $1DFC	; /
	RTS
PltfrmUp:
	LDA #$29	; \ Play sound
	STA $1DFC	; /
	LDA $3B			; \ Set platform length and layer 1 x position
	CMP #$02		; |
	BEQ .normal		; |
	LDA #$01		; |
	STA $3B			; |
	STA $2B			; |
	LDA #!pltfrmLnth1;|
	STA !pltfrmRAM	; /
	RTS
.normal
	LDA #$00		; \ Set platform length and layer 1 x position
	STA $3B			; |
	STA $2B			; |
	LDA #!pltfrmLnth; |
	STA !pltfrmRAM	; /
	RTS
PltfrmDn:
	LDA #$29	; \ Play sound
	STA $1DFC	; /
	LDA $3B				; \ Set platform length and layer 1 x position
	CMP #$01			; |
	BEQ PltfrmUp_normal	; |
	LDA #$02			; |
	STA $3B				; |
	STA $2B				; |
	LDA #!pltfrmLnth2	; |
	STA !pltfrmRAM		; /
	RTS

CheckContact:
	LDA !plyrX	; \ Player X pos low byte
	STA $00		; /
	STZ $08		; > Player X pos high byte
	LDA !plyrY	; \ Player Y pos low byte
	STA $01		; /
	STZ $09		; > Player Y pos high byte
	LDA #$0C	; \ Player width
	STA $02		; /
	STA $03		; > Player height
	STA $06		; > Sprite width
	STA $07		; > Sprite height

	LDA !X,x	; \ Sprite X pos low byte
	STA $04		; /
	STZ $0A		; > Sprite X pos high byte
	LDA !Y,x	; \ Sprite Y pos low byte
	STA $05		; /
	STZ $0B		; > Sprite Y pos high byte

	JSL $03B72B	; > Check for contact
	BCS .contact
	RTS
.contact
	LDA !sprTPE,x	; \ Load sprite type into X
	PHX : TAX		; /
	LDA Run,x : ASL : TAX	; > Get index for routine
	JSR (SprPntr,x)	; > Run routine
	PLX : PLA : PLA	; \ Kill sprite
	JMP KillSprite	; /

HandleSprite:
	CMP #$01	; > Input: A = Sprite state, X = Slot, Y = ??? (Probably OAM slot of last processed sprite)
	BNE .noInit
	INC
	STA !state,x
	LDA #$A0		; \ Get random value between 0x00 and 0xA0 (Amount of blocks in big platform)
	JSR RANDOM		; |
	CLC : ADC #$28	; | Add 0x28 to spawn them over the platform
	STA !X,x		; / and use it as x position 
	LDA.b #Prop-Tiles-1
	JSR RANDOM
	STA !sprTPE,x
	LDA #$FF
	STA !Y,x
	LDA.b #!maxSpeed-!minSpeed
	JSR RANDOM
	CLC : ADC #!minSpeed
	STA !YSpeed,x
.noInit
	JSR CheckContact
	TXA : ASL : ASL : TAY	; > Get OAM slot (sprite slot * 4 since finding a free OAM slot is way too slow)
	LDA !X,x	; \ Store X/Y position directly as the screen is at 0,0 anyway
	STA $0200,y	; |
	LDA !Y,x	; |
	STA $0201,y	; /
	LDA !sprTPE,x	; \ Load tile/properties depending on sprite type
	PHX : TAX		; |
	LDA Tiles,x		; |
	STA $0202,y		; |
	LDA Prop,x		; |
	AND #%11001111	; |
	STA $0203,y		; |
	PLX				; /
	LDA #$02		; \ Set size to 16x16
	STA $0420,x		; /
	JSR SPRupdateY	; > Update sprite Y position
	RTS

SetMissed:
	PLX
KillSprite:
	LDA #$00		; \ Zero out sprite tables for current sprite
	STA !state,x	; |
	STA !X,x		; |
	STA !Y,x		; |
	STA !XSpeed,x	; |
	STA !YSpeed,x	; /
	RTS

InitSpriteTables:
	LDA #$00
	LDX #!sprAmount-1	; \ Initialize all sprite tables
-	STA !state,x		; |
	STA !X,x			; |
	STA !Y,x			; |
	STA !XSpeed,x		; |
	STA !YSpeed,x		; |
	STA !sprTPE,x		; |
	STA !Xfrac,x		; |
	STA !Yfrac,x		; |
	DEX : BPL -			; /
	STZ !plyrY			; \ Initialize player addresses
	STZ !plyrDIST		; |
	STZ !plyrX			; |
	STZ !plyrYSpeed		; |
	STZ !plyrXSpeed		; |
	STZ !plyrXTmp		; |
	STZ !plyrSTATE		; /
	RTS

; Taken from all.log adjusted to work with new ram addresses.
SPRupdateYGravity:
	JSR SPRupdateY  	 ; Update Y position
	LDA !YSpeed,x	;\ 
	CLC				;| add 03 if sprite address is 00(case1), 01 if it's not (case2)
	ADC $019030		;|
	STA !YSpeed,x	;/ to Ysp
	BMI .return		; if that makes the sprite go up, 
	CMP $01902E		;\ if speed is slower than 40 for case1, or 10 for case2, 
	BCC .return		;/
	LDA $01902E		;\ 
	STA !YSpeed,x	;/ make that the speed
.return
	RTS

; Input:
;   X = Slot
SPRupdateY:
	PHX : TXA
	CLC : ADC #!sprAmount
	TAX : JSR SPRupdateX
	CLC : ADC #$10
	BCS SetMissed1
	PLX : RTS
SetMissed1:
	JMP SetMissed

; Input:
;   X = Slot
SPRupdateX:
	LDA !XSpeed,x
	BEQ .return				; If speed is 0, skip
	ASL                       ; \ Multiply speed by 16
	ASL                       ; | 
	ASL                       ; |
	ASL                       ; /
	CLC
	ADC !Xfrac,x
	STA !Xfrac,x
	PHP
	LDY.B #$00
	LDA !XSpeed,x		; Load X speed 
	LSR                       ; \  
	LSR                       ; |Divide speed by 16 
	LSR                       ; | 
	LSR                       ; /  
	CMP.B #$08
	BCC +
	ORA.B #$F0
	DEY
+	PLP
	ADC !X,x       ; \ Add value to X position
	STA !X,x       ; /
.return
	RTS				; Return

; Taken from the ASM code library
;============;
;RNG by 1024
;============;

RANDOM:
	PHX
	PHP
	SEP #$30
	PHA
	JSL $01ACF9
	PLX
	CPX #$FF
	BNE NORMALRT
	LDA $148B
	BRA ENDRANDOM
NORMALRT:
	INX
	LDA $148B
	STA $4202
	STX $4203
	NOP #4
	LDA $4217
ENDRANDOM:
	PLP
	PLX
	RTS