;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Rocket's fuel meter.
;;
;; Filled up by bubbles of fuel from the lab.
;; During launch, decrements until game over.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!yellowPoint	=	$20		; fuel at which meter turns yellow
!redPoint		=	$14		; fuel at which meter turns red
!flashPoint		=	$0C		; fuel at which meter starts flashing

!fuelFreq		=	$18		; how slowly to decrease the meter in phase 3

!destX			=	$E8		; position for the meter to move to in phase 2
!destY			=	$68		;  (position should differ from current by a multiple of 2)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
!fuelRAM	=	$1440
!fuelTimer	=	$1441
!gamePhase	=	$144E
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; code begins
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
	STZ !fuelRAM
	STZ !fuelTimer
	RTL
	
print "MAIN ",pc
	PHB
	PHK
	PLB
	JSR Main
	PLB
	RTL



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; main routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; $1504 - Flag for movement

Main:
	LDA $9D
	BEQ .notFrozen
	JMP Graphics
	
  .notFrozen
	LDA !gamePhase
	ASL
	PHX
	TAX
	REP #$20
	LDA RoutinePtrs,x
	STA $00
	SEP #$20
	PLX
	JMP ($0000)
	
RoutinePtrs:
	dw DoNothing
	dw Refuelling
	dw PrepLaunch
	dw Launching
	dw InFlight
	dw FollowCamera


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

DoNothing:
	JMP Graphics


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; If the fuel timer gets set, increase fuel meter by 4 pixels.

Refuelling:
	JSR Graphics
	LDA !fuelTimer
	BEQ .return
	DEC
	STA !fuelTimer
	LDX #$05
  .aniLoop
	CMP refuelAniTimer,x
	BEQ .done
	DEX
	BPL .aniLoop
	RTS
  .done
	INC !fuelRAM
  .return
	RTS

  refuelAniTimer:
	db $00,$04,$06,$07


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Prepare to move and finish refueling if necessary

PrepLaunch:
	JSR Graphics
	LDA.b #!destY-36
	SEC
	SBC $D8,x
	SEC
	ROR
	STA $AA,x
	LDA #!destX
	SEC
	SBC $E4,x
	LSR
	STA $B6,x
	JMP Refuelling


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Move the top-right.

Launching:
	LDA $1504,x
	BNE .doneMove
	LDA $D8,x
	CMP.b #!destY-36
	BEQ .doneMove
	JSL $018022		; update xpos
	JSL $01801A		; update ypos
	JSR Graphics
	RTS

  .doneMove
	LDA #$01
	STA $1504,x
	JSR FollowCamera
	RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Slowly decrease. If low on fuel, flash.

InFlight:
	LDA !fuelRAM		;\ 
	PHA					;|
	CMP #!flashPoint	;|
	BCS +				;|
	LDA $14				;|
	AND #$0C			;| Make it flash when low on fuel.
	BNE +				;|
	STZ !fuelRAM		;|
  +						;|
	JSR FollowCamera	;|
	PLA					;|
	STA !fuelRAM		;/
	
	LDA !fuelTimer
	DEC
	BPL .noDecrease
	DEC !fuelRAM
	LDA #!fuelFreq
  .noDecrease
	STA !fuelTimer
	RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Stay on-screen in a fixed position.

FollowCamera:
	LDA $1C
	STA $D8,x
	LDA $1D
	STA $14D4,x
	LDA $1B
	STA $14E0,x
	
	LDA #!destX
	STA $00
	LDA #!destY
	STA $01
	LDY $15EA,x
	JSR FixedGFX
	RTS



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Tiles:
	db $48,$49,$4A,$4B
	db $58,$59,$5A,$5B

!fullTile	=	$68		; Tile to use when completely full
!tbTile		=	$69		; Tile to use as the top/bottom
!tileCount	=	$07		; Number of tiles in the middle

Graphics:
	JSR GetDrawInfo
FixedGFX:
	PHX
	LDA $00					;\ 
	STA $0300,y				;|
	STA $0304,y				;|
	LDA $01					;|
	STA $0301,y				;|
	SEC						;|
	SBC #$08				;| Set up the top/bottom of the meter.
	STA $01					;|
	SBC.b #!tileCount*8		;|
	STA $0305,y				;|
	LDA #!tbTile			;|
	STA $0302,y				;|
	STA $0306,y				;|
	LDA #$21				;|
	STA $0303,y				;|
	ORA #$80				;|
	STA $0307,y				;/
	TYA
	CLC
	ADC #$08
	TAY
	
	LDX #$2B				;\ 
	LDA !fuelRAM			;|
	CMP #!yellowPoint		;|
	BCS .gotPalette			;|
	LDX #$25				;| Get palette for the meter.
	CMP #!redPoint			;|
	BCS .gotPalette			;|
	LDX #$29				;|
  .gotPalette				;|
	STX $03					;/
	
	LDA #$08
	STA $02
  .tileLoop
	LDA $00
	STA $0300,y
	LDA $01
	STA $0301,y
	SEC
	SBC #$08
	STA $01
	LDA $03
	STA $0303,y
	
	LDA !fuelRAM		;\ 
	SEC					;|
	SBC $02				;|
	LDX #!fullTile		;|
	BCS .doneTile		;|
	CLC					;|
	ADC #$08			;|
	BCC .empty			;|
	TAX					;| Set the tile for the meter.
	LDA Tiles,x			;|
	TAX					;|
	BRA .doneTile		;|
  .empty				;|
	LDX Tiles			;|
  .doneTile				;|
	TXA					;|
	STA $0302,y			;/
	
	INY #4
	LDA $02
	CLC
	ADC #$08
	STA $02
	CMP.b #!tileCount+1*8
	BCC .tileLoop
	
	PLX
	LDA.b #!tileCount+2
	LDY #$00
	JSL $01B7B3
	RTS
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; common subroutines
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; GetDrawInfo
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Table1:              db $0C,$1C
Table2:              db $01,$02

GetDrawInfo:
	STZ $186C,x
	STZ $15A0,x
	LDA $E4,x
	CMP $1A
	LDA $14E0,x
	SBC $1B
	BEQ OnscreenX
	INC $15A0,x
OnscreenX:
	LDA $14E0,x
	XBA
	LDA $E4,x
	REP #$20
	SEC
	SBC $1A
	CLC
	ADC.w #$0040
	CMP #$0180
	SEP #$20
	ROL A
	AND #$01
	STA $15C4,x
	BNE Invalid

	LDY #$00
	LDA $1662,x
	AND #$20
	BEQ OnscreenLoop
	INY
OnscreenLoop:
	LDA $D8,x
	CLC
	ADC Table1,y
	PHP
	CMP $1C
	ROL $00
	PLP
	LDA $14D4,x
	ADC #$00
	LSR $00
	SBC $1D
	BEQ OnscreenY
	LDA $186C,x
	ORA Table2,y
	STA $186C,x
OnscreenY:
	DEY
	BPL OnscreenLoop
	LDY $15EA,x
	LDA $E4,x
	SEC
	SBC $1A
	STA $00
	LDA $D8,x
	SEC
	SBC $1C
	STA $01
	RTS
	
Invalid:
	PLA
	PLA
	RTS