;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Rocket sprite.
;;
;; Stationary on the ground; after time runs out, launches and player assumes control.
;;  Once the fuel runs out, flips upside down and plummets back to Earth.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!shakeSFX	=	$01			; SFX for shaking during launch
!shakePort	=	$1DF9

!maxYSpeed	=	$A0			; Maximum vertical speed
!maxXSpeed	=	$20			; Maximum horizontal speed
!xAccel		=	$04			; How fast the ship accelerates horizontally

!sputter	=	$0A			; Fuel at which the rocket should start sputtering
!smokeSFX	=	$01			; SFX for the smoke
!smokePort	=	$1DF9

!failSFX	=	$28			; SFX for not having any fuel from the start
!failPort	=	$1DFC

!fallSFX	=	$26			; SFX for falling at the end
!fallPort	=	$1DF9

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
!fuelRAM	=	$1440
!fuelTimer	=	$1441
!lifeCount	=	$1448
!gameFailed	=	$1449
!gamePhase	=	$144E
!phaseTimer	=	$144F
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; code begins
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
print "INIT ",pc
	LDA $D8,x
	CLC
	ADC #$09
	STA $D8,x
	LDA $14D4,x
	ADC #$00
	STA $14D4,x
	RTL
	
print "MAIN ",pc
	PHB
	PHK
	PLB
	JSR Main
	PLB
	RTL



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; main routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; $1540 - Vertical acceleration timer
;; $1602 - Animation frame


Main:
	LDA $9D
	BEQ .spriteIsRunning
	JMP Graphics

  .spriteIsRunning
	LDA !gamePhase
	ASL
	PHX
	TAX
	REP #$20
	LDA RoutinePtrs,x
	STA $00
	SEP #$20
	PLX
	JMP ($0000)
	
RoutinePtrs:
	dw DoNothing
	dw Refuelling
	dw PrepLaunch
	dw Launching
	dw Flight
	dw GameOver


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

DoNothing:
	JMP Graphics


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Shake briefly after receiving fuel.

Refuelling:
	LDA !fuelTimer
	BNE .shake
	JSR Graphics
	BRA .done
  .shake
	JSR ShakeRocket
  .done
	RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Shake and play a sound.

PrepLaunch:
	JSR ShakeRocket
	LDA $14
	AND #$01
	BNE .return
	LDA #!shakeSFX
	STA !shakePort
  .return
	RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Start acceleration upwards and center on-screen.
;;  If you didn't have any fuel at all... well, you fail.

Launching:
	JSR ShakeRocket
	
	LDA !fuelRAM
	BEQ .youFailed
	
	JSR Accelerate
	LDA #$00
	LDY $E4,x
	CPY #$78
	BEQ .centered
	LDA #$FC
  .centered
	STA $B6,x
	
	JSL $018022				; update xpos
	JSL $01801A				; update ypos
	
	JSR ScreenTrack
	
	LDA $14
	AND #$07
	BNE .noSmoke
	JSR SpawnSmoke
  .noSmoke
	RTS


  .youFailed
	JSR SpawnSmoke
	LDA #!failSFX
	STA !failPort
	RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Give control to the player and interact with things.

Flight:
	LDA $1540,x				;\ 
	SEC						;|
	SBC #$03				;|
	BPL +					;| Increase acceleration speed.
	LDA #$00				;|
  +							;|
	STA $1540,x				;|
	JSR Accelerate			;/
	
	JSR FlightController
	JSL $018022				; update xpos
	JSL $01801A				; update ypos
	
	LDA $14D4,x				;\ 
	CMP #$02				;|
	BCS +					;|
	LDA #$0D				;| Loop screen #$01 to screen #$0D, right side.
	STA $14D4,x				;|
	LDA #$01				;|
	STA $14E0,x				;/
  +
	JSR ScreenTrack
	
	LDA $E4,x
	CMP #$10
	BCS .notLeftEdge
	INC $E4,x
	BRA .clearSpeed
  .notLeftEdge
	CMP #$E0
	BCC .notRightEdge
	DEC $E4,x
  .clearSpeed
	STZ $B6,x
  .notRightEdge
	
	JSR SpriteInteract
	JSL $019138				; block interaction
	
	LDA #$07
	LDY !fuelRAM
	CPY #!sputter
	BCS .noSputter
	PHY
	JSR ShakeRocket
	PLY
	LDA #$1F
	CPY.b #!sputter/2
	BCS .smoke
	LDA #$3F
	BRA .smoke
  .noSputter
	PHA
	JSR Graphics
	JSR FireGraphics
	PLA
  .smoke
	AND $14
	BNE .return
	JSR SpawnSmoke
	LDA #!smokeSFX
	STA !smokePort
  .return
	RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Flip upside down and fall.

FlipAni:
	db $01,$02,$81,$80

GameOver:
	LDA !gameFailed
	BNE .done
	
	LDA $AA,x
	BMI .limitSpeed
	CMP #$40
	BPL .noAccel
	BRA .accel
  .limitSpeed
	CMP #$C0		;\ 
	BCS .accel		;| Limit speed here just so that it doesn't fly over the top of the screen
	LDA #$C0		;|
	STA $AA,x		;/
  .accel
	INC $AA,x
	BMI .noAccel
	INC $AA,x
  .noAccel
  
	LDA $D8,x
	SEC
	SBC $1C
	CMP #$F0
	BCC .noErase
	STZ $14C8,x
	LDA #!fallSFX
	STA !fallPort
  .noErase
  
	JSL $01801A		; update ypos
	
	LDA $AA,x
	BPL .done
	CMP #$F0
	BMI .done
	CLC
	ADC #$10
	LSR #2
	TAY
	LDA FlipAni,y
	STA $1602,x
  .done
	JSR Graphics
	RTS



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; subroutines
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Shake the rocket and draw graphics.
ShakeRocket:
	LDA $1A
	PHA
	LDA $14
	AND #$02
	LSR
	SEC
	SBC $1A
	STA $1A
	JSR Graphics
	LDA !gamePhase
	CMP #$03
	BCC +
	JSR FireGraphics
  +
	PLA
	STA $1A
	RTS



;; Accelerate the rocket.
Accelerate:
	LDA $AA,x
	BEQ .doAccel
	CMP #!maxYSpeed
	BCC .dontAccel
  .doAccel
	LDA $1540,x
	BNE .dontAccel
	DEC $AA,x
	LDA $AA,x
	EOR #$FF
	INC
	STA $1540,x
  .dontAccel
	RTS



;; Spawn a smoke sprite below the rocket.
!smokeOffY	=	$2A

SpawnSmoke:
	LDY #$03
  -
	LDA $17C0,y
	BEQ +
	DEY
	BPL -
	RTS
  +
	LDA #$01
	STA $17C0,y
	LDA #$18
	STA $17CC,y
	LDA $D8,x
	CLC
	ADC #!smokeOffY
	STA $17C4,y
	LDA $E4,x
	STA $17C8,y
	RTS



;; Make the screen follow the sprite.
;; The code for this is... a bit weird looking, probably.
ScreenTrack:
	STZ $55
	LDA $14D4,x
	XBA
	LDA $D8,x
	REP #$20
	SEC
	SBC #$00A0
	STA $00
	LDA $14E0,x		;\ 
	AND #$00FF		;| Ignore if shifted 
	BNE +			;/
	LDA $00
	SEC
	SBC $1C
	CMP #$FFFB		; keep the screen scrolling even if below this point
	BCS +
	LDA $1464
	DEC
	STA $00
  +
	LDA $00
	STA $1464
	SEP #$20
	LDA $14E0,x
	STA $1463
	RTS



;; Bind Mario's controls to the rocket.
FlightController:
	LDA $15
	AND #$03
	BEQ .noInput
	CMP #$02
	BEQ .left
  .right
	LDA $B6,x
	CMP #!maxXSpeed
	BPL .return
	CLC
	ADC #!xAccel
	BRA .setSpeed
	
  .left
	LDA $B6,x
	CMP.b #!maxXSpeed^$FF+1
	BMI .return
	SEC
	SBC #!xAccel
  .setSpeed
	STA $B6,x
  .return
	RTS

  .noInput
	LDA $B6,x
	BEQ .return
	BPL .left
	BRA .right
	


;; Interact with the flying 1up sprite.
SpriteInteract:
	LDY #$09
  .loop
	LDA $14C8,y
	CMP #$08
	BNE .skip
	LDA $009E,y
	CMP #$7F
	BNE .skip
	JSL $03B69F
	PHX
	TYX
	JSL $03B6E5
	PLX
	JSL $03B72B
	BCS .found
  .skip
	DEY
	BPL .loop
	RTS

  .found
	LDA #$00
	STA $14C8,y
	LDA #$08
	JSL $02ACE5
	INC !lifeCount
	RTS



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; graphics
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; setting high bit of $1602 will flip upside down

Tilemap:
	db $00,$02,$20,$22,$40,$42,$FF	; 00: vertical
	db $06,$08,$24,$26,$28,$44,$46	; 01: diagonal
	db $0A,$0C,$0E,$2A,$2C,$2E,$FF	; 02: horizontal
XOff:
	db $F8,$08,$F8,$08,$F8,$08,$FF
	db $00,$10,$F0,$00,$10,$F0,$00
	db $F0,$00,$10,$F0,$00,$10,$FF
YOff:
	db $F8,$F8,$08,$08,$18,$18,$FF
	db $F8,$F8,$08,$08,$08,$18,$18
	db $00,$00,$00,$10,$10,$10,$FF

FireTilemap:
	db $4C,$4E
!fireXOff	=	$00
!fireYOff	=	$28
!fireOAM	=	$00

Graphics:
	JSR GetDrawInfo
	
	PHX
	STZ $02			;\ flip upside if bit 7 set
	LDA $1602,x		;|
	BPL .noFlip		;|
	DEC $02			;|
	PHA				;|
	LDA $01			;|
	CLC				;|
	ADC #$08		;|
	STA $01			;/
	PLA
  .noFlip
	AND #$03		;\ multiply frame by 7
	STA $03			;|
	ASL				;|
	STA $04			;|
	ASL				;|
	CLC				;|
	ADC $03			;|
	ADC $04			;|
	TAX				;/
	
	LDA #$06
	STA $03
  .rocketLoop
	LDA Tilemap,x	;\ set tile; if #$FF, end loop
	CMP #$FF		;|
	BEQ .done		;|
	STA $0302,y		;/
	
	LDA XOff,x		;\ set x offset
	CLC				;|
	ADC $00			;|
	STA $0300,y		;/
	
	LDA YOff,x		;\ set y offset, y-flip if necessary
	EOR $02			;|
	SEC				;|
	SBC $02			;|
	CLC				;|
	ADC $01			;|
	STA $0301,y		;/
	
	LDA $02			;\ set yxppccct
	AND #$80		;|
	ORA #$2F		;|
	STA $0303,y		;/
	
	INX
	INY #4
	DEC $03
	BPL .rocketLoop

  .done
	PLX
	LDA #$06		;\ draw 6 or 7 tiles
	SEC				;|
	ADC $02			;|
	LDY #$02		;|
	JSL $01B7B3		;/
	RTS




FireGraphics:
	JSR GetDrawInfo
	
	PHX
	LDY #!fireOAM
	LDA #!fireYOff		;\ set y offset
	CLC					;|
	ADC $01				;|
	STA $201,y			;/
	LDA #!fireXOff		;\ set x offset
	CLC					;|
	ADC $00				;|
	STA $200,y			;/
	LDA $14				;\ set tile 
	AND #$04			;|
	LSR #2				;|
	TAX					;|
	LDA FireTilemap,x	;|
	STA $202,y			;/
	LDA #$15			;\ set yxppccct
	STA $203,y			;/
	TYA					;\ set size
	LSR #2				;|
	TAY					;|
	LDA #$02			;|
	STA $420,y			;/
  .return
	PLX
	RTS



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; common subroutines
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; GetDrawInfo
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Table1:              db $0C,$1C
Table2:              db $01,$02

GetDrawInfo:
	STZ $186C,x
	STZ $15A0,x
	LDA $E4,x
	CMP $1A
	LDA $14E0,x
	SBC $1B
	BEQ OnscreenX
	INC $15A0,x
OnscreenX:
	LDA $14E0,x
	XBA
	LDA $E4,x
	REP #$20
	SEC
	SBC $1A
	CLC
	ADC.w #$0040
	CMP #$0180
	SEP #$20
	ROL A
	AND #$01
	STA $15C4,x
	BNE Invalid

	LDY #$00
	LDA $1662,x
	AND #$20
	BEQ OnscreenLoop
	INY
OnscreenLoop:
	LDA $D8,x
	CLC
	ADC Table1,y
	PHP
	CMP $1C
	ROL $00
	PLP
	LDA $14D4,x
	ADC #$00
	LSR $00
	SBC $1D
	BEQ OnscreenY
	LDA $186C,x
	ORA Table2,y
	STA $186C,x
OnscreenY:
	DEY
	BPL OnscreenLoop
	LDY $15EA,x
	LDA $E4,x
	SEC
	SBC $1A
	STA $00
	LDA $D8,x
	SEC
	SBC $1C
	STA $01
	RTS
	
Invalid:
	PLA
	PLA
	RTS