;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Level code for the game.
;; All main handling goes on in here.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!manual		=	$7FC070

!bubbleSpr	=	$02		; Sprite number of the fuel bubble

!phase0time	=	$C0		; Length of phase 0
!phase1time =	$77		; Length of phase 1 (divided by 4)
!phase2time	=	$40		; Length of phase 2
!phase5time	=	$A0		; Length of phase 5 (divided by 4)

!second		=	$0F		; Length of one "second" in phase 1 (divided by 4).

!coinTime	=	$80		; When to show the coin count.
!lifeTime	=	$70		; When to show the life count.
!decTime	=	$60		; When to start decrementing the coin counter.
!coinLife	=	$1E		; How many coins will give Mario a life.

!blkSFX		=	$06		; SFX for flipping through the blocks at the start.
!blkPort	=	$1DFC
!startSFX	=	$05		; SFX for the timer starting
!startPort	=	$1DF9
!tickSFX	=	$13		; SFX for the timer ticking down.
!tickPort	=	$1DF9
!launchSFX	=	$17		; SFX for the rocket launching off the ground
!launchPort	=	$1DFC

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
!fuelRAM	=	$1440		; amount of fuel the ship has
!fuelTimer	=	$1441		; timer for increasing/decreasing the meter
!blkCount	=	$1442		; number of blocks hit so far, also used for intro ani
!coinTrack	=	$1443		; scorecard uses this to count towards 1ups
!coinCount	=	$1446		; number of coins collected in the game (16-bit)
!lifeCount	=	$1448		; number of lives collected in the game
!gameFailed	=	$1449		; flag to indicate you really suck at this game
!gamePhase	=	$144E		; current phase of the game
!phaseTimer	=	$144F		; timer for various parts of each phase
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; LOAD
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ...originally there was planned to be random level generation in here, but...
;; Instead is just used to clear out some data.

load:
	LDA #$00
	STA !manual
	STA !manual+1
	STA !manual+2
	STZ !gamePhase
	RTL



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INIT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Initialize phase timer.

init:
	LDA $13				;\ 
	STA $148B			;| Give random (...ish) initial seed.
	EOR $14				;|
	STA $148C			;/
	
	LDA #$03			;\ 
	JSR RandGen			;| Randomly set an initial block.
	TAX					;|
	STA !blkCount		;/

	LDA #!phase0time
	STA !phaseTimer
	RTL



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAIN
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

main:
	LDA $9D
	ORA $13D4
	BEQ +
	RTL
  +
	LDA !gamePhase
	ASL
	TAX
	JMP (MainPtrs,x)

MainPtrs:
	dw InitWait
	dw BlockGame
	dw LaunchWait
	dw Launching
	dw Flight
	dw GameOver


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Phase 0 - Opening presentation
;; This handles all the animation for that.

InitWait:
	LDA !phaseTimer
	CMP #!phase0time-20
	BCS .noArrows
	AND #$0C
	BEQ .noArrows
	JSR DrawArrows
  .noArrows
	LDX #$00
	JSR DrawReadyStart
	
	LDA !phaseTimer
	CMP #!phase0time-30
	BCS .notNext
	LDY #$07
	CMP #!phase0time-70
	BCS +
	LDY #$0F
	CMP #!phase0time-90
	BCS +
	LDY #$1F
  +
	TYA
	AND !phaseTimer
	BNE .notNext
	LDX !blkCount
	LDA #$00
	STA !manual,x
	DEX
	BPL +
	LDX #$02
  +
	STX !blkCount
	LDA #$01
	STA !manual,x
	LDA #!blkSFX
	STA !blkPort
  .notNext
	
	JSR DisableInput
	DEC !phaseTimer
	BNE .return
	INC !gamePhase
	LDA #!phase1time
	STA !phaseTimer
	LDA #$01
	STA !blkCount
	LDA #!startSFX
	STA !startPort
  .return
	RTL


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Phase 1 - Block game portion
;; This handles the timer sprite.

BlockGame:
	LDA $14
	AND #$03
	BNE .noDec
	DEC !phaseTimer
  .noDec
	LDA !phaseTimer
	BEQ .endPhase
	
	CMP #!second*5
	BCC .drawNum
	LDX #$04
	JSR DrawReadyStart
	BRA .return
  .drawNum
	LDX #$FF
  .countSecs
	INX
	SEC
	SBC #!second
	BPL .countSecs
	CMP.b #$FF
	BNE +
	LDA #!tickSFX
	STA !tickPort
  +
	JSR DrawTimer
  .return
	RTL

  .endPhase
	INC !gamePhase
	LDA #!phase2time
	STA !phaseTimer
	RTL


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Phase 2 - Waiting to launch (rocket shakes)
;; Just a timer.

LaunchWait:
	JSR DisableInput
	DEC !phaseTimer
	BPL .return
	INC !gamePhase
	LDA #!launchSFX
	STA !launchPort
  .return
	RTL


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Phase 3 - Rocket centers and levels starts scrolling
;; Most of this is handled by the rocket, the main code is just waiting.

Launching:
	LDA !fuelRAM
	BNE .goodJob
	LDA #$05
	STA !gamePhase
	LDA #!phase5time
	STA !phaseTimer
	INC !gameFailed
	RTL
	
  .goodJob
	JSR DisableInput
	JSR RetainMario
	LDA $1D
	CMP #$0C
	BCS .return
	INC !gamePhase
  .return
	RTL


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Phase 4 - Player assumes control of rocket.
;; This code spawns flying 1ups every 4 screens.

!mushSlot	=	$00		; only one spawns at a time anyway so WHATEVER

Flight:
	JSR RetainMario
	
	LDA $1D
	AND #$03
	BNE .noMush
	LDA $1C
	AND #$80
	BNE .noMush
	JSR SpawnMushroom
  .noMush
	
	LDA !fuelRAM
	BPL .return
	STZ !fuelRAM
	INC !gamePhase
	LDA #!phase5time
	STA !phaseTimer
  .return
	RTL


SpawnMushroom:
	LDX #!mushSlot
	LDA $14C8,x
	BNE .return
	LDA #$01
	STA $14C8,x
	LDA #$7F
	STA $9E,x
	JSL $01ACF9
	AND #$7F
	CLC
	ADC #$20
	STA $E4,x
	LDA $1B
	STA $14E0,x
	LDA $1C
	STA $D8,x
	LDA $1D
	STA $14D4,x
	JSL $07F7D2		; init sprite
  .return
	RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Phase 5 - Rocket flips and the score card shows.
;; This code specifically handles the score card and timing.

CoinStripe:		; (c) COINS x 0__
	db $51,$8A,$00,$19
	db $2E,$38,$FC,$38
	db $0C,$38,$18,$38,$12,$38,$17,$38,$1C,$38
	db $FC,$38,$26,$38,$FC,$38
	db $00,$38,$FC,$38,$FC,$38
	db $FF

LifeStripe:		; (m) LIVES x 0__
	db $51,$CA,$00,$19
	db $76,$38,$FC,$38
	db $15,$38,$12,$38,$1F,$38,$0E,$38,$1C,$38
	db $FC,$38,$26,$38,$FC,$38
	db $00,$38,$FC,$38,$FC,$38
	db $FF

GameOver:
	JSR DisableInput
	JSR RetainMario

	LDY #$03
	LDA !gameFailed
	BEQ +
	LDY #$01
  +
	TYA
	AND $14
	BNE +
	DEC !phaseTimer
  +
	LDA !phaseTimer
	BNE ScoreCard
	LDA #$0B
	STA $0100
	RTL


ScoreCard:
	LDA !phaseTimer
	CMP #!coinTime
	BCC .writeCoins
	RTL

  .writeCoins
	REP #$30			;\ 
	LDA $7F837B			;|
	TAX					;|
	SEP #$20			;|
	LDY #$0000			;|
  .coinLoop				;|
	LDA CoinStripe,y	;|
	STA $7F837D,x		;|
	INX					;| Write the coin count to the screen.
	INY					;|
	CMP #$FF			;|
	BNE .coinLoop		;|
	DEX					;|
	TXA					;|
	STA $7F837B			;|
	STX $03				;|
	REP #$20			;|
	LDA !coinCount		;|
	JSR StoreNum		;/

	LDA !phaseTimer		;\ 
	CMP #!decTime		;|
	BCS .noDec			;|
	LDA $14				;| Decrease coin count.
	AND #$01			;|  Every 20 coins, give a life.
	BNE .noDec			;|
	LDA !coinCount		;|
	BEQ .noDec			;|
	LDA #$01			;|
	STA $1DFC			;|
	INC !coinTrack		;|
	DEC !coinCount		;|
	BEQ +				;|
	INC !coinTrack		;|
	DEC !coinCount		;|
  +						;|
	LDA !coinTrack		;|
	CMP #!coinLife		;|
	BCC .noDec			;|
	SEC					;|
	SBC #!coinLife		;|
	STA !coinTrack		;|
	INC !lifeCount		;|
	INC $0DBE			;|
	LDA #$05			;|
	STA $1DFC			;|
  .noDec				;/

	LDA !phaseTimer
	CMP #!lifeTime
	BCS .return

	REP #$30			;\ 
	LDA $7F837B			;|
	TAX					;|
	SEP #$20			;|
	LDY #$0000			;|
  .lifeLoop				;|
	LDA LifeStripe,y	;|
	STA $7F837D,x		;|
	INX					;|
	INY					;| Write the life count to the screen.
	CMP #$FF			;|
	BNE .lifeLoop		;|
	DEX					;|
	TXA					;|
	STA $7F837B			;|
	STX $03				;|
	REP #$20			;|
	LDA !lifeCount		;|
	AND #$00FF			;|
	JSR StoreNum		;/
  .return
	SEP #$10
	RTL



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Subroutines
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Store max value + 1 to A before jumping here; result is in A
RandGen:
	PHA
	JSL $01ACF9
	STA $4202
	PLA
	STA $4203
	NOP #4
	LDA $4217
	RTS



;; Disable player input
DisableInput:
	STZ $15
	STZ $16
	STZ $17
	STZ $18
	RTS



;; Prevent Mario from dying off the bottom of the level.
RetainMario:
	REP #$20
	LDA $80
	CMP #$00E4
	BCC +
	LDA $1C
	CLC
	ADC #$00E4
	STA $96
	LDA $1A
	CLC
	ADC #$0080
	STA $94
	SEP #$20
	
	LDA $16
	AND #$1F
	STA $16
	STZ $18
	
	LDA #$01			;\ 
	STA $185C			;| Prevent block/sprite interaction.
	INC $154C+!mushSlot	;/
  +
	SEP #$20
	RTS



;; Stores the coin/life values to the VRAM table. A should contain the 16-bit value to use.
StoreNum:
	SEP #$10			;\ 
	STA $4204			;|
	LDY #$64			;|
	STY $4206			;|
	JSR Waste24Cycles	;|
	LDX $4214			;|
	STX $00				;|
	LDA $4216			;| Convert to a three-digit decimal.
	STA $4204			;| Output is stored in $00-$02.
	LDY #$0A			;|
	STY $4206			;|
	JSR Waste24Cycles	;|
	LDY $4214			;|
	STY $01				;|
	LDY $4216			;|
	STY $02				;|
	REP #$10			;/
	SEP #$20

	LDX $03
	LDA $00
	AND #$0F
	BEQ .skipHunds
	STA $7F837D-6,x
	BRA .tens
  .skipHunds
	DEX #2

  .tens
	LDA $00
	ORA $01
	BEQ .skipTens
	LDA $01
	STA $7F837D-4,x
	BRA .ones
  .skipTens
	DEX #2

  .ones
	LDA $02
	STA $7F837D-2,x
	RTS

;; Wastes 24 cycles, for division.
Waste24Cycles:
	NOP #12
	RTS



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; GFX routines
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Draw the arrows above the blocks.

!arrowTile	=	$A0		; Tile for the arrow sprite
!arrowProp	=	$29		;  and properties

!arrowOAM	=	$80

DrawArrows:
	LDA #$20
	STA $00
	LDY #$00
  .loop
	LDA $00
	STA $0200+!arrowOAM,y
	CLC
	ADC #$20
	STA $00
	LDA #$58
	STA $0201+!arrowOAM,y
	LDA #!arrowTile
	STA $0202+!arrowOAM,y
	LDA #!arrowProp
	STA $0203+!arrowOAM,y
	TYA
	LSR #2
	TAX
	LDA #$02
	STA !arrowOAM/4+$0420,x
	INY #4
	CPY #$0C
	BNE .loop
	RTS



;; Draw the "Ready"/"Start!" text. Index to which should be in X (00 or 04).
ReadyTiles:
	db $A2,$A4,$A6,$A8	; READY
	db $C2,$C4,$C6,$C8	; START!

!readyProp	=	$29
!startProp	=	$2B
!readyOAM	=	$90

DrawReadyStart:
	STZ $01
	LDA #$00
	CPX #$04
	BNE +
	JSL $01ACF9
	AND #$01
	STA $01
	JSL $01ACF9
	AND #$01
  +
	CLC
	ADC #$28
	STA $00
	LDY #$00
  .loop
	LDA $00
	STA $0200+!readyOAM,y
	CLC
	ADC #$10
	STA $00
	LDA #$30
	CLC
	ADC $01
	STA $0201+!readyOAM,y
	LDA ReadyTiles,x
	STA $0202+!readyOAM,y
	LDA #!readyProp
	CPX #$04
	BCC +
	LDA #!startProp
  +
	STA $0203+!readyOAM,y
	PHY
	TYA
	LSR #2
	TAY
	LDA #$02
	STA !readyOAM/4+$0420,y
	PLY
	INX
	INY #4
	CPY #$10
	BNE .loop
	RTS



;; Draw the timer for the block portion. X should contain the number of seconds (-1).
TimerTiles:
	db $88,$86,$84,$82,$80
TimerProps:
	db $33,$39,$35,$37,$3B

!timerOAM	=	$90

DrawTimer:
	JSL $01ACF9
	AND #$01
	CLC
	ADC #$40
	STA $0200+!timerOAM
	JSL $01ACF9
	AND #$01
	CLC
	ADC #$30
	STA $0201+!timerOAM
	LDA TimerTiles,x
	STA $0202+!timerOAM
	LDA TimerProps,x
	STA $0203+!timerOAM
	LDA #$02
	STA !timerOAM/4+$0420
	RTS